'''
Gtf format annotation processing
Copyright (c) 2016 Peng Zhang <zhpn1024@163.com>
'''

def add_chr(chr):
  if len(chr) <= 3 and (chr.isdigit() or chr in ('X','Y',)): chr = 'chr' + chr
  elif chr in ('M','MT','MtDNA','mitochondrion_genome','Mito') : chr = 'chrM' 
  elif len(chr) == 2 and chr[0].isdigit(): chr = 'chr' + chr
  elif chr in ('I','II','III','IV','V','VI','VII','VIII','IX','XI','XII','XIII','XIV','XV','XVI'): chr = 'chr' + chr
  return chr
def rm_chr(chr):
  if chr[3:].isdigit() or chr in ('chrX','chrY'): chr = chr[3:]
  elif chr == 'chrM' : chr = 'MT'
  return chr
def changechr(chr):
  '''change between two chr versions
  '''
  if chr.isdigit() or chr in ('X','Y','M'): return 'chr' + chr
  elif chr == 'MT' : return 'chrM'
  elif chr == 'chrM' : return 'MT'
  elif chr[0:3] == 'chr' : return chr[3:]
  else : return chr
def cmp3(a, b):
  ''' cmp for python3'''
  return (a > b) - (a < b)

class Exon:
  '''exon in gtf format, 0 based, similar as bed
  '''
  def __init__(self, lst, gff = False, addchr = False):
    self.chr, self.strand, self.start, self.stop = lst[0], lst[6], int(lst[3]) - 1, int(lst[4])
    if addchr and self.chr[0:3] != 'chr' :
      if self.chr.isdigit() or self.chr in ('X','Y','M',): self.chr = 'chr' + self.chr
      elif self.chr == 'MT' : self.chr = 'chrM' 
    self.type, self.score, self.attrstr = lst[2], lst[5], lst[8]
    self.gff, self.addchr, self.frame = gff, addchr, lst[7]
    self.lst = lst
    self.exons = [self]
  def __repr__(self):
    return self.chr + ':'+str(self.start)+'-'+str(self.stop)+':'+self.strand
  def __str__(self, sep = '\t', new = False):
    if not new : return sep.join(self.lst)
    lst = [self.chr, self.lst[1], self.type, str(self.start+1), str(self.stop), str(self.score), self.strand, str(self.frame), self.attrstr]
    return sep.join(lst)
  def string(self, sep = '\t', new = False):
    return self.__str__(sep = sep, new = new)
  def short_str(self):
    return "%s:%d-%d:%s" % (self.chr, self.start, self.stop, self.strand)
  def __cmp__(self, other):
    return cmp(self.chr,other.chr) or cmp(self.start,other.start) or cmp(self.stop,other.stop)
  def cmp(self, other):
    return cmp3(self.chr,other.chr) or cmp3(self.start,other.start) or cmp3(self.stop,other.stop)
  def __eq__(self, other):
    return self.cmp(other) == 0
  def __lt__(self, other):
    return self.cmp(other) < 0
  def __gt__(self, other):
    return self.cmp(other) > 0
  def __len__(self): 
    return self.stop - self.start
  def attr(self, key):
    '''get attributes in the last column
    '''
    s = self.attrstr
    if self.gff : 
      p1 = s.find(key+'=')
      if p1 < 0 : p1 = s.find(key+':')
      if p1 < 0 : return ''
      else : p1 += len(key) #+ 1
      p2 = s.find(';', p1)
      if p2 == -1 : p2 = None
      if s[p1] == '=' : return s[p1+1:p2]
      elif s[p1] == ':' : return s[p1+1:p2].split(',')[0]
      else : return ''
    p1 = s.find(key + ' ')
    if p1 < 0 : return ''
    else : p1 += len(key) + 1
    p2 = s.find(';', p1)
    if p2 == -1 : p2 = None
    try: return eval(s[p1:p2])
    except: return s[p1:p2]
  @property
  def length(self):
    return len(self)
  def cdna_length(self): 
    return len(self)
  def is_reverse(self): 
    return self.strand=='-'
  @property
  def anti_strand(self):
    if self.strand == '+': return '-'
    elif self.strand == '-': return '+'
    else : return '.'
  @property
  def gid(self):
    try : return self.gid_c # gid cache
    except : 
      if self.gff : self.gid_c = self.attr('GeneID')
      else : self.gid_c = self.attr('gene_id')
    return self.gid_c
  @property
  def tid(self):
    try : return self.tid_c
    except : 
      self.tid_c = self.attr('transcript_id')
      if self.gff: # and self.tid_c == '':
        if self.lst[2] in ('transcript', 'mRNA', 'tRNA', 'rRNA'):
          self.tid_c = self.id
        else:
          p = self.attr('Parent')
          if p.startswith('rna-'): self.tid_c = p[4:]
          elif p.startswith('gene-'): self.tid_c = p[5:]
          elif p.startswith('rna'): self.tid_c = p
          elif p.startswith('gene'): self.tid_c = p
      return self.tid_c
  @property
  def symbol(self):
    try : return self.sym_c
    except : 
      self.sym_c = self.attr('gene_name')
      if self.gff and self.sym_c == '': self.sym_c = self.attr('gene')
    return self.sym_c
  @property
  def id(self):
    try: return self._id
    except:
      self._id = self.attr('exon_id')
      if self._id == '': self._id = self.attr('ID')
      return self._id
  @id.setter
  def id(self, value):
    self._id = value
  @property
  def end5(self): #5' end, all bed
    if self.strand != '-' : return self.start
    else : return self.stop
  @property
  def end3(self): #3' end, all bed
    if self.strand != '-' : return self.stop
    else : return self.start
  @property
  def genetype(self):
    a = self.attr('gene_biotype')
    if a != '' : return a
    a = self.attr('gene_type')
    if a != '' : return a
    if self.tid[0:3] == 'NM_' : return "protein_coding"
    return a
  def short_str(self):
    return "%s:%d-%d:%s" % (self.chr, self.start, self.stop, self.strand)
  def __call__(self, **args):
    '''New exon that modified from self.
    '''
    lst = self.lst[:]
    new = Exon(lst, self.gff, self.addchr)
    for k in args:
      new.__dict__[k]=args[k]
    return new
  def __sub__(self, other):
    return sub(self, other)
  def intersect(self, other):
    return intersect(self, other)
  def union(self, other):
    return union(self, other)
  def is_contain(self, p, strict = False): #if i in exon
    if self.start <= p <= self.stop :
      if not strict : return True
      else : return p != self.end3
    else : return False
  def is_exon(self, p, strict = False): 
    return self.is_contain(p, strict = strict)
  def is_intron(self, p, strict = False): # False
    return False
  def is_upstream(self, p, strict = False) :
    if self.strand != '-' : return p < self.start
    else : return p > self.stop
  def is_downstream(self, p, strict = False) :
    if self.strand != '-' : 
      if strict : return p >= self.stop
      else : return p > self.stop
    else : 
      if strict : return p <= self.start
      else : return p < self.start
  def flank_pos(self, p, flank = 100):
    if flank < 0: return None
    if self.start - flank <= p <= self.start:
      if self.strand != '-': return p - self.start # upstream
      else: return self.start - p + self.cdna_length()
    if self.stop <= p <= self.stop + flank:
      if self.strand != '-': return p - self.stop + self.cdna_length()
      else: return self.stop - p # upstream
    return None

  def cdna_pos(self, p, strict = False, flank = 0): # exon only
    if self.is_contain(p, strict): return abs(p-self.end5)
    elif flank > 0: return self.flank_pos(p, flank)
    else: return None
  def genome_pos(self, p, bias = 1): # exon only
    m = self.cdna_length()
    if p < 0 or p > m: return None
    if p == 0 : return self.end5
    if p == m: return self.end3
    if not self.is_reverse() : return self.start + p
    else : return self.stop - p
  def copy(self, other):
    '''copy everything from other exon
    '''
    self.chr, self.strand, self.start, self.stop = other.chr, other.strand, other.start, other.stop
    self.type, self.score, self.attrstr = other.type, other.score, other.attrstr
    #self.attr = other.attr
    self.gff, self.addchr, self.frame = other.gff, other.addchr, other.frame
    self.lst = other.lst
  #@property
  #def exons(self):
    #return [self]
  def cds_start(self, cdna = False): return None
  def cds_stop(self, cdna = False): return None
    
def attr(s, gff = False):
  '''get attr dict from attr string
  '''
  if type(s) == dict: return s
  if gff : 
    l = s.strip(';').split(';')
    a = {}
    for att in l:
      l2 = att.split('=')
    #print att, l2
      a[l2[0]] = l2[1]
      if l2[0] == "Dbxref" :
        l3 = l2[1].split(',')
        for att2 in l3:
          l4 = att2.split(':')
          a[l4[0]] = l4[1]
    return a
  l = s.strip(';').split('; ')
  a = {}
  for att in l:
    att = att.strip()
    p = att.find(' ')
    a[att[0:p]] = eval(att[p+1:])
  return a
def attrstr(a) :  # Not tested!
  s = ''
  for k in a :
    s += k + ' '
    if type(a[k]) == str : s += '"' + a[k] + '"'
    else : s += str(a[k])
    s += '; '
  return s
def tup2com(t): #tuple to comma string
  if type(t)==str: return t
  return ','.join(map(str,t))+','

class gtfTrans(Exon):
  '''gtf transcript
  '''
  def __init__(self, lst, gff = False, addchr = False): 
    Exon.__init__(self, lst, gff, addchr)
    if self.tid != '': self.id = self.tid
    elif self.id != '': self.tid_c = self.id
    #self.type = lst[1]
    self.exons = []
    self.cds = []
    self.utr = []
    self.start_codon = None
    self.stop_codon = None
    self.other = []
  def add_exon(self, e): 
    if e.strand != self.strand : 
      print('Wrong exon strand: {} {} {} {}'.format(e.gid, e.tid, e.strand, self.strand))
      e.strand = self.strand
    if e.type == 'exon': self.exons.append(e)
    elif e.type == 'CDS': self.cds.append(e)
    elif e.type == 'UTR': self.utr.append(e)
    elif e.type == 'start_codon': self.start_codon = e
    elif e.type == 'stop_codon': self.stop_codon = e
    else: self.other.append(e)
    #self.check()
  def __repr__(self):
    s = self.symbol + " transcript_id " + self.id + ', ' + str(len(self.exons)) + " exons, " + Exon.__repr__(self)
    return s
  def __str__(self, sep = '\t', new = False):
    arr = self.exons + self.cds + self.utr + self.other
    if self.start_codon is not None : arr.append(self.start_codon)
    if self.stop_codon is not None : arr.append(self.stop_codon)
    arr.sort(reverse = self.is_reverse())
    s = ''
    if self.type == 'transcript' : s += Exon.__str__(self, new = new) + '\n'
    for e in arr : 
      s += e.__str__(sep = sep, new = new) + '\n'
    return s.strip()
  def string(self, sep = '\t', new = False):
    return self.__str__(sep = sep, new = new)
  def allRecords(self):
    arr = self.exons + self.cds + self.utr + self.other
    if self.start_codon is not None : arr.append(self.start_codon)
    if self.stop_codon is not None : arr.append(self.stop_codon)
    arr.sort(reverse = self.is_reverse())
    if self.type == 'transcript' : arr[0:0] = [self]
    return arr
  def genePredStr(self, geneName = True, extended = False, slice = None):
    '''generate genepred format
    '''
    lst = []
    if geneName : lst += [self.gid]
    start, stop, thickstart, thickstop = self.start, self.stop, self.thick_start, self.thick_stop
    if slice is None :
      exons = self.exons
    else :
      if start < slice[0] : start = slice[0]
      if thickstart < slice[0] : thickstart = slice[0]
      if stop > slice[1] : stop = slice[1]
      if thickstop > slice[1] : thickstop = slice[1]
      exons = []
      for e in self.exons:
        if e.stop <= slice[0] : continue
        if e.start >= slice[1] : continue
        exons.append(e(start = max(e.start, slice[0]), stop = min(e.stop, slice[1])))
    lst += [self.id, self.chr, self.strand, start, stop, thickstart, thickstop, len(exons)]
    exonStarts = [x.start for x in exons]
    exonStops = [x.stop for x in exons]
    exonStarts.sort()
    exonStops.sort()
    lst += [tup2com(exonStarts), tup2com(exonStops)]
    return '\t'.join(map(str, lst))
  def check(self, final = True):
    if final and len(self.exons) == 0:
      if len(self.cds) > 0: self.exons += self.cds
    for e in self.exons:
      if e.start < self.start: self.start = e.start
      if e.stop > self.stop: self.stop = e.stop
    self.exons.sort(reverse = self.is_reverse())
    self.cds.sort(reverse = self.is_reverse())
  #@property
  def cds_start(self, cdna = False): 
    if self.start_codon is None and len(self.cds) == 0 : return None
    if self.start_codon is None :
      self.cds.sort(reverse = self.is_reverse())
      gs = self.cds[0].end5
      if self.stop_codon is not None : 
        cds2 = self.cds_stop(cdna = True)
        cds1 = self.cdna_pos(gs)
        cs = cds2 - (cds2 - cds1) // 3 * 3
        if cdna : return cs
        else : return self.genome_pos(cs, 1)
      try : frame = int(self.cds[0].frame)
      except : frame = 0
      if frame > 0 : 
        cs = self.cdna_pos(gs) + frame
        if cdna : return cs
        else : return self.genome_pos(cs, 1)
      if cdna : return self.cdna_pos(gs)
      else : return gs
    sc = self.start_codon
    if len(sc) != 3 : 
      for e in self.exons:
        if sc.end5 == e.end5 : 
          cs = self.cdna_pos(sc.end3) - 3
          if cdna : return cs
          else : return self.genome_pos(cs, 1)
        if sc.end3 == e.end3 :
          break
      else : 
        print('Start codon error : %s %s %s' % (self.gid, self.id, sc.short_str()))
    gs = sc.end5
    if cdna : return self.cdna_pos(gs)
    else : return gs
  def cds_stop(self, cdna = False):
    if self.stop_codon is None and len(self.cds) == 0 : return None
    if self.stop_codon is None :
      self.cds.sort(reverse = self.is_reverse())
      gs = self.cds[-1].end3
      if self.start_codon is not None : # determine by paired start codon
        cds1 = self.cds_start(cdna = True)
        cds2 = self.cdna_pos(gs)
        cs = cds1 + (cds2 - cds1) // 3 * 3
        if cdna : return cs
        else : return self.genome_pos(cs, 0)
      try : frame = int(self.cds[-1].frame) # determine by frame
      except : frame = None
      if frame is not None : 
        cds1 = self.cdna_pos(self.cds[-1].end5) + frame
        cds2 = self.cdna_pos(gs)
        cs = cds1 + (cds2 - cds1) // 3 * 3
        if cdna : return cs
        else : return self.genome_pos(cs, 1)
      if cdna : return self.cdna_pos(gs) #cs
      else : return gs # self.genome_pos(cs, 0)
    sc = self.stop_codon # Determine by stop codon
    if len(sc) != 3 : 
      for e in self.exons:
        if sc.end3 == e.end3 : 
          cs = self.cdna_pos(sc.end5) + 3
          if cdna : return cs
          else : return self.genome_pos(cs, 0)
        if sc.end5 == e.end5 :
          break
      else : 
        print('Stop codon error : %s %s %s' % (self.gid, self.id, sc.short_str()))
    gs = sc.end3
    if cdna : return self.cdna_pos(gs)
    else : return gs
  @property
  def thick_start(self):
    if self.is_reverse(): s = self.cds_stop(cdna = False)
    else : s = self.cds_start(cdna = False)
    if s is None : return self.start
    else : return s
  @property
  def thick_stop(self):
    if self.is_reverse(): s = self.cds_start(cdna = False)
    else : s = self.cds_stop(cdna = False)
    if s is None : return self.start
    else : return s
  @property
  def itemRgb(self):
    return '0,0,0'
  @property
  def blockCount(self):
    return len(self.exons)
  @property
  def blockSizes(self):
    es = self.exons[:]
    if self.is_reverse() : es.sort()
    return ','.join([str(len(e)) for e in es])
  @property
  def blockStarts(self):
    es = self.exons[:]
    if self.is_reverse() : es.sort()
    return ','.join([str(e.start - self.start) for e in es])
  def cdna_length(self): 
    l = 0
    for e in self.exons:
      l += len(e)
    return l
  def cds_length(self): 
    try : return self.cds_stop(cdna = True) - self.cds_start(cdna = True) ##
    except: return 0
  @property
  def introns(self):
    if hasattr(self, '_introns') : return self._introns
    introns = []
    last = -1
    for i, e in enumerate(self.exons):
      if last >= 0 : 
        lst = [last, e.end5]
        lst.sort()
        it = Exon([self.chr,'','intron',lst[0]+1,lst[1],'',self.strand,'',self.attrstr], gff=self.gff)
        it.id += '_intron{}'.format(i)
        introns.append(it)
      last = e.end3
    self._introns = introns
    return introns
  def cdna_pos(self, p, strict = False, flank = 0):
    '''if strict is True, the 3' end of exon will be considered as not in the transcript,
    if strict is False, 3' end of exon will be considered as start of the next exon, 
    or transcript end (self.cdna_length()) if in the last exon.
    if flank is not 0, TSS and TTS flanking regions are returned with negative distance and
    cDNA length + distance
    '''
    if not self.is_contain(p, strict) :
      if flank <= 0: return None
      else: return self.flank_pos(p, flank)
    pos = 0
    for e in self.exons:
      if e.is_upstream(p, strict) : return None
      if e.is_contain(p, strict) : return pos + abs(p - e.end5) # e.start <= p <= e.stop:
      pos += len(e)
    return None
  def is_exon(self, p, strict = False):
    if not self.is_contain(p, strict) : return False
    for e in self.exons:
      if e.is_upstream(p, strict) : return False
      if e.is_contain(p, strict) : return True # e.start <= p <= e.stop:
    return False
  def genome_pos(self, p, bias = 1):
    '''if bias is 1, the splice junction will be mapped to 5' end of downstream exon,
    if bias is 0, the splice junction will be mapped to 3' end of the upstream exon.
    '''
    m = self.cdna_length()
    if p < 0 or p > m: return None
    if p == 0 : return self.end5
    if p == m: return self.end3
    p1 = p
    pos = self.start
    for e in self.exons:
      if len(e) - p1 >= bias:
        if self.is_reverse():
          pos = e.stop - p1
        else:
          pos = p1 + e.start
        return pos
      else:
        p1 -= len(e)
    return None
  def copytrans(self, o):
    self.exons, self.cds, self.utr, self.start_codon, self.stop_codon, self.others = o.exons, o.cds, o.utr, o.start_codon, o.stop_codon, o.others
    
class gtfGene(Exon):
  '''gtf gene
  '''
  def __init__(self, lst, gff = False, addchr = False):
    Exon.__init__(self, lst, gff, addchr)
    self.id = self.gid
    self.trans = []
    #self.type = lst[1]
  def add_trans(self, tr):
    if self.strand in ('+', '-') and tr.strand != self.strand :
      print('Inconsistent trans strand: {} {} {} {}'.format(tr.gid, tr.tid, tr.strand, self.strand))
      #tr.strand = self.strand
      self.strand = '.'
    self.trans.append(tr)
    #self.check()
  def __repr__(self):
    s = "gene_id " + self.id + ', ' + str(len(self.trans)) + " transcripts, " + Exon.__repr__(self)
    for t in self.trans:
      s += '\n\t' + t.__repr__()
    return s
  def check(self):
    for t in self.trans:
      t.check()
      if t.start < self.start: self.start = t.start
      if t.stop > self.stop: self.stop = t.stop
  def merge_trans(self):
    '''generate a new transcript that merge all transcript exons in the gene
    '''
    es = []
    merge = gtfTrans(self.lst)
    for t in self.trans:
      es += t.exons
    es.sort()
    me = es[0]
    for i in range(1, len(es)):
      if me.stop >= es[i].start:
        me = union(me, es[i])[0]
      else: 
        merge.add_exon(me)
        me = es[i]
    merge.add_exon(me)
    merge.check()
    return merge
  def max_cds(self):
    return selectMaxCDS(self)
    
def load_gtf(fin, filt = [], gff = False, addchr = False, verbose = False):
  genes = {}
  trans = {}
  symbol = {}
  dfilt = {}
  if len(filt) > 0: 
    for gid in filt : dfilt[gid] = True
  chr = ''
  for l in fin:
    if l[0] == '#' : continue
    lst=l.strip().split('\t')
    if len(lst) < 2 : continue
    if lst[0] != chr :
      ks = list(dfilt.keys())
      for k in ks : 
        if k in genes or k in trans or k in symbol : del(dfilt[k])
        #if genes[gid].symbol in dfilt : del(dfilt[genes[gid].symbol])
      chr = lst[0]
      if verbose : print(chr)
    #for k in dfilt:
      #if not dfilt[gid] : continue
      #if lst[8].find(k) >= 0 : break
    #else : continue
    if lst[2] == 'region' : continue
    e = Exon(lst, gff, addchr)
    if len(filt) > 0 and e.gid not in dfilt and e.tid not in dfilt and e.symbol not in dfilt: 
      continue
    if lst[2] == 'gene':
      g = gtfGene(lst, gff, addchr)
      if g.id in genes: g.trans = genes[g.id].trans
      genes[g.id] = symbol[g.symbol] = g
    elif lst[2] == 'transcript':
      t = gtfTrans(lst, gff, addchr)
      if t.gid not in genes:
        g = gtfGene(lst, gff, addchr)
        genes[g.id] = symbol[g.symbol] = g
      if t.id in trans: trans[t.id].copy(t)
      else : 
        genes[t.gid].add_trans(t)
        trans[t.id] = t
    else:
      e = Exon(lst, gff, addchr)
      if e.gid not in genes:
        g = gtfGene(lst, gff, addchr)
        genes[g.id] = symbol[g.symbol] = g
      if e.tid not in trans:
        t = gtfTrans(lst, gff, addchr)
        genes[t.gid].add_trans(t)
        trans[t.id] = t
      trans[e.tid].add_exon(e)
  for g in genes : genes[g].check()
  return genes, trans

def fetch_gtf(fin, gid = '', tid = '', gff = False, addchr = False):
  genes = {}
  trans = {}
  if gid == '' and tid == '' : return genes, trans
  ch = ''
  for l in fin:
    if l[0] == '#' : continue
    lst=l.strip().split('\t')
    if len(lst) < 2 : continue
    if ch != '' and ch != lst[0] : break
    if gid != '' and lst[8].find(gid) < 0 : continue
    if tid != '' and lst[8].find(tid) < 0 : continue
    if lst[2] == 'region' : continue
    e = Exon(lst, gff, addchr)
    if e.tid != tid  and e.gid != gid : continue
    if e.gid == '' : continue
    if lst[2] == 'gene':
      g = gtfGene(lst, gff, addchr)
      if g.id in genes: g.trans = genes[g.id].trans
      if g.id == gid : genes[g.id] = g
    elif lst[2] == 'transcript':
      t = gtfTrans(lst, gff, addchr)
      if t.id != tid  and t.gid != gid : continue 
      if t.gid not in genes:
        g = gtfGene(lst, gff, addchr)
        genes[g.id] = g
      if t.id in trans: trans[t.id].copy(t)
      else : 
        genes[t.gid].add_trans(t)
        trans[t.id] = t
    else:
      if e.gid not in genes:
        g = gtfGene(lst, gff, addchr)
        genes[g.id] = g
      if e.tid not in trans:
        t = gtfTrans(lst, gff, addchr)
        genes[t.gid].add_trans(t)
        trans[t.id] = t
      trans[e.tid].add_exon(e)
    ch = g.chr
  for g in genes : genes[g].check()
  return genes, trans

def gtfgene_iter(fin, filt = [], gff = False, addchr = False, chrs = None, verbose = False):
  genes, genes2, trans, gidlist = {}, {}, {}, [] # genes2: genes with headline, for better output
  dfilt = {}
  if len(filt) > 0: 
    for k in filt : dfilt[k] = 1
  chr = ""
  for l in fin:
    if l[0] == '#' : continue
    lst=l.strip().split('\t')
    if len(lst) < 2 : continue
    if lst[2] == 'region' : continue
    if lst[0] != chr:
      if chrs is not None and lst[0] not in chrs and changechr(lst[0]) not in chrs: 
        #print('{}\t{}\t{}'.format(lst[0], changechr(lst[0]), chrs))
        continue
      if verbose : print(lst[0])
      for gid in gidlist:
        if gid in genes : 
          genes[gid].check()
          yield genes[gid]
      genes, genes2, trans, gidlist = {}, {}, {}, []
      chr = lst[0]
    e = Exon(lst, gff, addchr)
    if len(filt) > 0 and e.gid not in dfilt: 
        continue
    if e.gid not in genes : # check gene output
      dgs = []
      for gid in genes2 : 
        if e.start > genes2[gid].stop : 
          genes2[gid].check()
          yield genes2[gid]
          dgs.append(gid)
      for gid in dgs:
        del genes2[gid]
        del genes[gid]
    if lst[2] == 'gene':
      g = gtfGene(lst, gff, addchr)
      if g.id in genes: g.trans = genes[g.id].trans # Already exists due to disorder
      else : gidlist.append(g.id)
      genes[g.id] = g
      genes2[g.id] = g
      #if g.id not in gidlist: gidlist.append(g.id)
    elif lst[2] in ('transcript', 'mRNA', 'tRNA', 'rRNA'):
      t = gtfTrans(lst, gff, addchr)
      if t.gid not in genes:
        g = gtfGene(lst, gff, addchr)
        genes[g.id] = g
        gidlist.append(g.id)
      if t.id in trans: trans[t.id].copy(t) # Already exists due to disorder
      else : 
        genes[t.gid].add_trans(t)
        trans[t.id] = t
    else:
      if e.gid == '' or e.tid == '' : continue
      if e.gid not in genes:
        g = gtfGene(lst, gff, addchr)
        genes[g.id] = g
        gidlist.append(g.id)
      if e.tid not in trans:
        t = gtfTrans(lst, gff, addchr)
        genes[t.gid].add_trans(t)
        trans[t.id] = t
      trans[e.tid].add_exon(e)
  for gid in gidlist:
    if gid not in genes : continue
    genes[gid].check()
    yield genes[gid]

def selectMaxCDS(g): 
  '''select transcript with max CDS length
  '''
  maxlen, mt = 0, None
  for t in g.trans:
    cds1, cds2 = t.cds_start(cdna = True), t.cds_stop(cdna = True) 
    try : cdslen = cds2 - cds1 
    except : continue
    if cdslen % 3 != 0 : continue
    if cdslen > maxlen : maxlen, mt = cdslen, t
  return mt
def gtftransSelectIter(fin, select = selectMaxCDS, **kwargs):
  for g in gtfgene_iter(fin, **kwargs):
    t = select(g)
    if t is not None : yield t
  
def gtftrans_iter(fin, filt = [], gff = False, addchr = False, chrs = None, verbose = False): # To be update
  genes, trans, trans2, tidlist = {}, {}, {}, []
  dfilt = {}
  if len(filt) > 0: 
    for k in filt : dfilt[k] = 1
  chr = ""
  for l in fin:
    if l[0] == '#' : continue
    lst=l.strip().split('\t')
    if len(lst) < 2 : continue
    if lst[2] == 'region' : continue
    if lst[0] != chr:
      if chrs is not None and lst[0] not in chrs and changechr(lst[0]) not in chrs: continue
      if verbose : print(lst[0])
      for tid in tidlist:
        if tid in trans : 
          trans[tid].check()
          yield trans[tid]
      genes, trans, trans2, tidlist = {}, {}, {}, []
      chr = lst[0]
    if lst[2] == 'gene': continue ## not gene structure
    e = Exon(lst, gff, addchr)
    if len(filt) > 0 and e.tid not in dfilt: 
        continue
    if e.tid not in trans : # check gene output
      dgs = [] # genes to delete
      for tid in trans2 : 
        if e.start > trans2[tid].stop : 
          trans2[tid].check()
          yield trans2[tid]
          dgs.append(tid)
      for tid in dgs:
        del trans2[tid]
        del trans[tid]
    if lst[2] == 'transcript':
      t = gtfTrans(lst, gff, addchr)
      if len(filt) > 0 and t.tid not in dfilt: 
        continue
      if t.id in trans: trans[t.id].copy(t)
      else : 
        #genes[t.gid].add_trans(t)
        trans[t.id] = t
        trans2[t.id] = t
        tidlist.append(t.id)
    else:
      #e = exon(lst, gff, addchr)
      if e.tid not in trans:
        t = gtfTrans(lst, gff, addchr)
        #genes[t.gid].add_trans(t)
        trans[t.id] = t
        tidlist.append(t.id)
        #print e.attr['transcript_id'], t.id
      trans[e.tid].add_exon(e)
  for tid in tidlist:
    if tid not in trans : continue
    trans[tid].check()
    yield trans[tid]
    #for t in genes[gid].trans:
      #yield t
  #return genes, trans

def sub(a, b): # a - b
  if a.chr != b.chr : return [a]
  if a.start >= b.stop or a.stop <= b.start : return [a]
  out = []
  if a.start < b.start : out.append(a(stop = b.start))
  if b.stop < a.stop: out.append(a(start = b.stop))
  return out

def intersect(a, b): # a & b
  out = []
  if a.chr != b.chr : return out
  if a.start >= b.stop or a.stop <= b.start : return out
  out.append(a(start = max(a.start, b.start), stop = min(a.stop, b.stop)))
  return out

def union(a, b): # a U b
  if a.chr != b.chr or a.start > b.stop or a.stop < b.start :
    return [a, b]
  return [a(start = min(a.start, b.start), stop = max(a.stop, b.stop))]




'''
Statistics tools
Copyright (c) 2016 Peng Zhang <zhpn1024@163.com>
'''

import math
from scipy.stats import nbinom, chisquare # chisqprob
from scipy.stats import chi2
from scipy.special import betaln, betainc, gammaln
#logarr = [None] # log(N)
#logsumarr = [0] # log(N!)
def logsum(n): # log(N!)
  return gammaln(n+1)
def logarr_ext(n) : #, logarr = logarr, logsumarr = logsumarr): # prepare log values
  global logarr, logsumarr
  l = len(logarr)
  if l < n + 1 : 
    logarr += [None] * (n + 1 - l)
    logsumarr += [None] * (n + 1 - l)
    for i in range(l, n + 1):
      logarr[i] = math.log(i)
      #print i, logarr[i]
      logsumarr[i] = logsumarr[i-1] + logarr[i]
  return logarr, logsumarr
def betainc1(a, b, x):
  if a == 0: return 1
  return betainc(a, b, x)
def data_count(data):
  total, cnt = 0, 0
  for k in data:
    total += k * data[k]
    cnt += data[k]
  return total, cnt
def mean_var(data):
  total, tsq, cnt = 0, 0, 0
  for k in data:
    total += k * data[k]
    cnt += data[k]
    tsq += k * k * data[k]
  mean = float(total) / cnt
  var = float(tsq) / cnt - mean ** 2
  return mean, var
def load_data(arr):
  data = {}
  for k in arr:
    if k not in data : data[k] = 0
    data[k] += 1
  return data
def fisher_method(ps):
  n, fs = 0, 0
  for p in ps:
    if p is None : continue
    if p == 0 : return 0.0, -1 ###
    fs += - 2 * math.log(p)
    n += 1
  fp = chi2.sf(fs, 2 * n) # chisqprob(fs, 2 * n)
  return fp, fs

def combination_log(n, k, show = False): 
  '''N choose K, log combination number, NATURAL LOG
  '''
  n, k = int(n), int(k)
  if n < 0 : return None
  if k > n or k < 0: return None #None is log(0)
  #logarr_ext(n)
  lpr = logsum(n) - logsum(k) - logsum(n-k) # logsumarr[n] - logsumarr[k] - logsumarr[n-k]
  return lpr

def ACprob(x, y, r = 1): 
  '''p(y|x) = r^y C(x+y,y) / (1+r)^(x+y+1) , r = N2/N1 by Audic and Claverie 
  '''
  lp = combination_log(x+y, x)
  lp += math.log(r) * y
  lp -= math.log(1 + r) * (1 + x + y)
  return math.exp(lp)
def ACtest(x, y, r = 1, alt = 'auto', double = True): # Diff expression test by Audic and Claverie 
  if alt in ('g', 'greater') : n1, n2, nr = x, y, r  # if x > y ?
  elif alt in ('l', 'less') : n1, n2, nr = y, x, 1.0/r
  elif x * r < y : n1, n2, nr = y, x, 1.0/r # n2 is smaller than n1
  else : n1, n2, nr = x, y, r
  pv = 0
  for i in range(n2 + 1):
    pv += ACprob(n1, i, nr)
    if double and pv >= 0.5 : return 1
    #print pv
  if double : pv *= 2 # Two tailed
  if pv > 1 : pv = 1
  return pv

def FCtest(x, y, r = 1, fc = 1.5, alt = 'auto', double = True): 
  '''Test whether exp diff > fold change (binom test). For data with no replicate|dispersion 
  '''
  n = x + y
  if y > 0 : fcr = 1.0 * x * r / y
  else : fcr = fc + 1 ###
  if alt in ('g', 'greater') and fcr <= fc : return 1
  elif alt in ('l', 'less') and fcr >= 1.0 / fc : return 1
  if 1.0 / fc <= fcr <= fc : return 1
  if fcr > fc : 
    p = fc / (fc + r)
    pv = binom_test(x, n, p = p, alt = "g")
  elif fcr < 1.0 / fc : 
    p = 1.0 / (1 + r * fc)
    pv = binom_test(x, n, p = p, alt = "l")
  if double : pv *= 2  # Doubling the smaller tail
  if pv > 1 : pv = 1
  return pv
    
def hypergeo_log(N, K, n, k): 
  return combination_log(K, k) + combination_log(N-K, n-k) - combination_log(N, n)

def hypergeo(N, K, n, k):
  if k < 0 or k < K+n-N: return 0
  if k > n or k > K: return 0
  lp = hypergeo_log(N, K, n, k)
  if lp is None : return 0
  return math.exp(lp)
  
def hypergeo_test(N, K, n, k, alt = 'two.tailed') :
  p = hypergeo(N, K, n, k)
  if alt in ('g', 'greater') :
    for i in range(k, min(K, n)) :
      p += hypergeo(N, K, n, i+1)
      if p > 1 : return 1
    return p
  elif alt in ('l', 'less') :
    for i in range(max(0, n + K - N), k) :
      p += hypergeo(N, K, n, i)
      if p > 1 : return 1
    return p
  p *= 2
  if p > 1 : p = 1
  return p

def binomial(n, k, p = 0.5, show=False):
  if k > n or k < 0: return 0
  if p < 0 : p = 0
  if p > 1 : p = 1
  if k * 2 > n:
    k = n - k
    p = 1 - p
  q = 1 - p
  pr = 1.0
  nk = n - k
  if k == 0 : return q ** n
  t = int(nk / k)
  if t * k < nk: t += 1
  qi = 0
  for i in range(k):
    pi = 1.0
    pi *= n - i
    pi /= k - i
    pi *= p
    for ti in range(t):
      if qi >= nk: break
      pi *= q
      qi += 1
    pr *= pi
    if show: print (pr)
  return pr
def binom_log(n, k, p = 0.5, show = False): #log probability value
  if n < 0 : return None
  if k > n or k < 0: return None #None is log(0)
  if p <= 0 :
    if k == 0 : return 0 # log(1)
    else : return None
  elif p >= 1 : 
    if k == n : return 0
    else : return None
  #q = 1 - p
  lpr = math.log(p) * k + math.log(1-p) * (n-k)
  lpr += combination_log(n, k)
  return lpr
def binom_test0(n, k, p = 0.5, alt = "g", log = True, show=False): 
  '''binomial test, no two sided yet!
  '''
  if not log : return binomTest0(n, k, p, alt, show) # if log, p are calculated with log values
  lpk = binom_log(n, k, p)
  if show : print (lpk)
  if lpk is None : 
    if alt[0] == 'g' and p >= 1: return 1
    if alt[0] != 'g' and p <= 0: return 1
    return 0
  elif lpk == 0 : return 1
  pv = math.exp(lpk) ### log reverse
  q = 1 - p
  lp = math.log(p)
  lq = math.log(q)
  if alt[0] == 'g':
    for i in range(k, n):
      lpk += lp + math.log(n - i) - lq - math.log(i + 1) #r = p * (n - i) / q / (i + 1)
      pv += math.exp(lpk)
  else:
    for i in range(k, 0, -1):
      lpk += lq + math.log(i) - lp - math.log(n - i + 1) #r = q * i / p / (n - i + 1)
      pv += math.exp(lpk)
  return pv
def binom_test(n, k, p = 0.5, alt = "g"):
  if alt[0] == 'l':
    return betainc1(n-k, k+1, 1-p)
  elif alt[0] == 'g':
    return betainc1(k, n-k+1, p)
  else:
    if k <= n * p:
      pv = betainc1(n-k, k+1, 1-p)
    else:
      pv = betainc1(k, n-k+1, p)
    return min(pv*2, 1)

def binomTest0(n, k, p = 0.5, alt = "g", show=False): # No two sided yet!
  '''binomial test no log version
  '''
  pk = binomial(n, k, p)
  if show : print (pk)
  if pk == 1 : return 1
  if pk == 0 : 
    if alt[0] == 'g' and p >= 1: return 1
    if alt[0] != 'g' and p <= 0: return 1
    return 0
  pv = pk
  q = 1 - p
  if alt[0] == 'g': 
    for i in range(k, n):
      #r = 1.0
      r = p * (n - i) / q / (i + 1)
      pk *= r
      pv += pk
  else:
    for i in range(k, 0, -1):
      r = q * i / p / (n - i + 1)
      pk *= r
      pv += pk
  return pv
class NegBinom: 
  '''Negative binomial distribution
  Number of 'failures' before 'r' 'successes' with success probability 'p'
  Note that when p is higher, the expect is lower, consistent with nbinom in scipy.stats.
  '''
  rMax = 1e8
  rMin = 1e-8
  Delta = 1e-8
  p_record = {}
  def __init__(self, r = 1.0, p = 0.5):
    self.p = p
    self.r = r
  @property
  def q(self):
    return 1 - self.p
  def expect(self):
    return self.q * self.r / self.p
  def variance(self):
    return self.expect() / self.p
  def logpmf(self, k = 0):
    return nbinom.logpmf(k, self.r, self.p)
  def pmf(self, k = 0):
    return nbinom.pmf(k, self.r, self.p)
  def cdf(self, k = 0):
    return nbinom.cdf(k, self.r, self.p)
  def pvalue(self, k = 0, record = True):
    if record : 
      key = (self.r, self.p)
      if key not in self.p_record : self.p_record[key] = {}
      elif k in self.p_record[key] : return self.p_record[key][k]
    if k == 0: p = 1
    else: p = betainc(k, self.r, self.q)
    '''
    p = 1 - nbinom.cdf(k-1, self.r, self.p)
    if p > self.Delta : 
      if record : self.p_record[key][k] = p
      return p
    p = nbinom.pmf(k, self.r, self.p)
    if p == 0 : 
      if record : self.p_record[key][k] = p
      return p
    ka = k + 1
    pa = nbinom.pmf(ka, self.r, self.p)
    p += pa
    while pa / p > self.Delta:
      #p += pa
      ka += 1
      pa = nbinom.pmf(ka, self.r, self.p)
      p += pa
    #p += pa'''
    if record : self.p_record[key][k] = p
    return p
      
  def estimate(self, data): #data dict value:counts
    '''estimate parameters from input data (dict type, value -> counts)
    '''
    total, cnt = data_count(data)
    rmax, rmin = self.rMax, self.rMin
    rmid = math.sqrt(rmax * rmin)
    while (rmax - rmin) / rmid >= self.Delta:
      #print rmax, rmin
      score = self.r_log_like_score(data, rmid)
      #print score, cnt, rmid
      if score > 0 : rmin = rmid
      elif score < 0 : rmax = rmid
      else : break
      #rmid = (rmax + rmin) / 2
      rmid = math.sqrt(rmax * rmin)
    self.r = rmid
    #self.p = total / (self.r * cnt + total)
    self.p = self.r / (self.r + 1.0*total/cnt)
    return self.r, self.p

  def log_likelihood(self, data):
    score = 0.0
    for k in data:
      score += data[k] * self.logpmf(k)
    return score
  def r_log_like_score(self, data, r = -1):
    '''log likelyhood score as function of r
    '''
    if r < 0 : r = self.r
    total, cnt = data_count(data)
    s1, d = 0, 0
    for i in range(max(data.keys()) + 1):
      if i in data : s1 += d * data[i]
      d += 1.0 / (r + i)
    score = s1 / cnt + math.log(r / (r + 1.0*total/cnt))
    return score
  def expected(self, k, size):
    ''' expected number under given sample size
    '''
    p = self.pmf(k)
    return size * p
  def estimate_truncated(self, data, size, max_iter = 1e4, nlike = 10, report = False): 
    '''estimate with truncated data. Not finished!
    '''
    total, cnt = data_count(data)
    lastllh = 0
    i = 0
    km = max(data)
    for j in range(int(max_iter)) :
      ps = 0
      for k in data : ps += self.pmf(k)
      #size = round(total / ps)
      k = 0
      # generate expected full data
      d = {}
      while k <= km or d[k-1] >= 1 : 
        if k in data : d[k] = data[k]
        else : d[k] = round(self.expected(k, size)) ## round?
        k += 1
      self.estimate(d)
      i += 1
      if i < nlike : continue
      i = 0
      llh = self.log_likelihood(d)
      diff = abs(2 * (llh - lastllh) / (llh + lastllh) / nlike)
      if diff < self.Delta : break
      if report : 
        s = '{} {} {} {} {} {}'.format(diff, llh, self.r, self.p, k, size)
        for i in range(15) : s += ' %d:%d' % (i, d[i])
        s += '...'
        print(s)
      lastllh = llh
    return self.r, self.p
  def estimate_by_012(self, n0, n1, n2, start = 0) : 
    '''estimate by first 3 numbers
    '''
    r1 = (start + 1) * float(n1) / n0 
    r2 =  (start + 2) * float(n2) / n1
    p1 = r2 - r1
    self.p = 1 - p1 # 
    self.r = (r1 - start * p1) / p1
    return self.r, self.p
  def fit_lower(self, data, nmax = 40, pth = 0.01, start = 0) : # in test, not used
    r, p = self.estimate_by_012(data[start], data[start+1], data[start+2], start = start)#, nlike=100)
    if r > 0 and 0 < p < 0.9 : 
      #self.r, self.p = r, p
      return r, p
    else : 
      import numpy as np
      from scipy.optimize import leastsq
      def res(p, x, y) : 
        return p[0] * x + p[1] - y
      m = max(data)
      if m > nmax : m = nmax # max
      s = data[start] + data[start+1] + data[start+2]
      y = [(start + 1) * float(data[start+1]) / data[start], (start + 2) * float(data[start+2]) / data[start+1]]
      w = [(data[start+1] * data[start])**0.5, (data[start+1] * data[start+2])**0.5]
      for i in range(start+3, m): 
        y.append(i * float(data.value(i)) / data.value(i-1))
        w.append((data[i] * data[i-1])**0.5)
        wm = min([wi for wi in w if wi > 0])
        w2 = [int(round(wi / wm)) for wi in w]
        s += data.value(i)
        xl = sum([[j] * w2[j] for j in range(i)], [])
        yl = sum([[y[j]] * w2[j] for j in range(i)], [])
        xa = np.array(xl, dtype = float)
        ya = np.array(yl, dtype = float)
        rst = leastsq(res, [1,1], args=(xa, ya))
        p = 1 - rst[0][0]
        r = rst[0][1] / rst[0][0]
        print (i, (r, p), len(xl), w)#, y
        if r <= 0 or p >= 1 or p <= 0 : continue
        if 0.2 < p < 1 : break
        self.r, self.p = r, p
        #f1 = self.pmf(i) / self.cdf(i)
        f1 = self.pmf(i+1) / (self.cdf(i+1) - self.cdf(start-1))
        f2 = self.pmf(i+2) / (self.cdf(i+2) - self.cdf(start-1))
        #pv1 = binom_test(data.value(i), s, p = f1, alt = 'g')
        pv1 = binom_test(data.value(i+1), s+data.value(i+1), p = f1, alt = 'g')
        pv2 = binom_test(data.value(i+2), s+data.value(i+1)+data.value(i+2), p = f2, alt = 'g')
        #for j in range(i) : print j, self.pmf(j),
        print (data.value(i+1), s+data.value(i+1), f1, 'pv1 =', pv1)
        print (data.value(i+2), s+data.value(i+1)+data.value(i+2), f2, 'pv2 =', pv2)
        #fp, fs = fisher_method([pv1, pv2])
        fp = max(pv1, pv2)
        #print fp
        if fp < pth : break #and pv2 < pth : break
      self.r, self.p = r, p
      return r, p
  def fit_linear(self, data, poisson = False, maxi = 40, minc = 3, start = 0, total = None, show = False) : # in test, not used
    if poisson : N2 = 0
    else : 
      if total is None : total = sum(data.values())
      N2 = 2.0 / total
    rate, rvar = {}, {}
    for i in data : 
      if i - 1 < start : continue
      if i-1 not in data or data[i-1] <= 0 : continue
      if i not in data or data[i] <= 0 : continue
      if data[i] < minc : break
      #if data.value(i-1) == 0 or data.value(i) == 0 : continue
      if i >= maxi : break
      rate[i] = float(i) * data[i] / data[i-1]
      rvar[i] = rate[i] ** 2 * (1.0 / data[i] + 1.0 / data[i] - N2)
    #print 'rate =', rate
    #print 'rvar =', rvar
    c1, c2, c3, c4, c5 = 0,0,0,0,0
    a, b = {}, {}
    r, p = {}, {}
    #ia = rate.keys()
    #ia.sort()
    ia = sorted(rate)
    for n, i in enumerate(ia) : 
      c1 += 1 / rvar[i]
      c2 += i / rvar[i]
      c3 += i * i / rvar[i]
      c4 += rate[i] / rvar[i]
      c5 += i * rate[i] / rvar[i]
      if n > 0 : 
        a[i] = (c3 * c4 - c2 * c5) / (c1 * c3 - c2 * c2)
        b[i] = (c2 * c4 - c1 * c5) / (c2 * c2 - c1 * c3)
        p[i] = 1 - b[i]
        r[i] = a[i] / b[i] + 1
        if show : print (i, a[i], b[i], r[i], p[i])
    self.r, self.p = r[i], p[i]
    return a, b, r, p, rate, rvar
    r, p = self.estimate_by_012(data[start], data[start+1], data[start+2], start = start)#, nlike=100)
    if r > 0 and 0 < p < 0.9 : 
      #self.r, self.p = r, p
      return r, p
    else : 
      import numpy as np
      from scipy.optimize import leastsq
      def res(p, x, y) : 
        return p[0] * x + p[1] - y
      m = max(data)
      if m > nmax : m = nmax # max
      s = data[start] + data[start+1] + data[start+2]
      y = [(start + 1) * float(data[start+1]) / data[start], (start + 2) * float(data[start+2]) / data[start+1]]
      w = [(data[start+1] * data[start])**0.5, (data[start+1] * data[start+2])**0.5]
      for i in range(start+3, m): 
        y.append(i * float(data.value(i)) / data.value(i-1))
        w.append((data[i] * data[i-1])**0.5)
        wm = min([wi for wi in w if wi > 0])
        w2 = [int(round(wi / wm)) for wi in w]
        s += data.value(i)
        xl = sum([[j] * w2[j] for j in range(i)], [])
        yl = sum([[y[j]] * w2[j] for j in range(i)], [])
        xa = np.array(xl, dtype = float)
        ya = np.array(yl, dtype = float)
        rst = leastsq(res, [1,1], args=(xa, ya))
        p = 1 - rst[0][0]
        r = rst[0][1] / rst[0][0]
        print (i, (r, p), len(xl), w) #, y
        if r <= 0 or p >= 1 or p <= 0 : continue
        if 0.2 < p < 1 : break
        self.r, self.p = r, p
        #f1 = self.pmf(i) / self.cdf(i)
        f1 = self.pmf(i+1) / (self.cdf(i+1) - self.cdf(start-1))
        f2 = self.pmf(i+2) / (self.cdf(i+2) - self.cdf(start-1))
        #pv1 = binom_test(data.value(i), s, p = f1, alt = 'g')
        pv1 = binom_test(data.value(i+1), s+data.value(i+1), p = f1, alt = 'g')
        pv2 = binom_test(data.value(i+2), s+data.value(i+1)+data.value(i+2), p = f2, alt = 'g')
        #for j in range(i) : print j, self.pmf(j),
        print (data.value(i+1), s+data.value(i+1), f1, 'pv1 =', pv1)
        print (data.value(i+2), s+data.value(i+1)+data.value(i+2), f2, 'pv2 =', pv2)
        #fp, fs = fisher_method([pv1, pv2])
        fp = max(pv1, pv2)
        #print fp
        if fp < pth : break #and pv2 < pth : break
      self.r, self.p = r, p
      return r, p
  def chisquare_test(self, data):
    total, cnt = data_count(data)
    obs, exs = [], []
    ob, ex = 0, 0
    i0 = 0
    for i in range(max(data.keys()) + 1) :
      if i in data : ob += data[i]
      ex += cnt * self.pmf(i)
      if ex >= 5 : 
        obs.append(ob)
        exs.append(ex)
        ob, ex = 0, 0
        i0 = i + 1
    ex = cnt * self.pvalue(i0)
    #obs.append(ob)
    #exs.append(ex)
    obs[-1] += ob
    exs[-1] += ex
    print (obs, exs, len(obs) - 1, len(exs), sum(obs), sum(exs))
    return chisquare(obs, exs)
  
class ZTNB(NegBinom):
  '''Zero truncated negative binomial
  '''
  def logpmf(self, k = 1):
    if k < 1 : return NegBinom.pmf(self, -1)
    p0 = NegBinom.pmf(self, 0)
    lp = NegBinom.logpmf(self, k)
    return lp - math.log(1 - p0)
  def pmf(self, k = 1):
    if k < 1 : return 0
    p0 = NegBinom.pmf(self, 0)
    p = NegBinom.pmf(self, k)
    return p / (1- p0)
  def expected_zeros(self, size):
    p0 = NegBinom.pmf(self, 0)
    return size * p0 / (1 - p0)
  def estimate(self, data, max_iter = 1e4, nlike = 10, report = False):
    '''estimate ZTNB parameters from input data (dict type, value -> counts)
    '''
    d1 = {}
    for i in data : 
      if i > 0 : d1[i] = data[i]
    total, cnt = data_count(d1)
    lastllh = 0
    i = 0
    for j in range(int(max_iter)) :
      ez = self.expected_zeros(cnt)
      d1[0] = ez
      NegBinom.estimate(self, d1)
      d1[0] = 0
      i += 1
      if i < nlike : continue
      i = 0
      llh = self.log_likelihood(d1)
      d = abs(2 * (llh - lastllh) / (llh + lastllh) / nlike)
      if d < self.Delta : break
      if report : print (d, llh, self.r, self.p, ez)
      lastllh = llh
    return self.r, self.p
  def pvalue(self, k = 1):
    if k <= 1 : return 1
    p0 = NegBinom.pmf(self, 0)
    p = NegBinom.pvalue(self, k)
    return p / (1- p0)
  def expect(self):
    p0 = NegBinom.pmf(self, 0)
    return NegBinom.expect(self) / (1 - p0)
  def variance(self):
    nb = NegBinom(self.r, self.p)
    p0 = nb.pmf(0)
    return (nb.variance() + nb.expect() ** 2) / (1 - p0) - self.expect() ** 2
    #return self.expect / self.p

class Poisson: 
  '''Poisson distribution
  '''
  lMax = 1e8
  lMin = 1e-8
  Delta = 1e-8
  def __init__(self, l = 1.0):
    self.l = l # lambda
  def expect(self):
    return self.l
  def variance(self):
    return self.l
  def logpmf(self, k = 0):
    logarr_ext(k)
    lpr = k * math.log(self.l) - self.l
    lpr -= logsumarr[k]
    return lpr
  def pmf(self, k = 0):
    return math.exp(self.logpmf(k))
  def cdf(self, k = 0): #, logarr = logarr):
    if k < 0 : return 0
    #logarr_ext(k)
    lpr = self.logpmf(0)
    logl = math.log(self.l)
    cdf = math.exp(lpr)
    for i in range(1, k + 1):
      lpr += logl - math.log(i) # logarr[i]
      cdf += math.exp(lpr)
    return cdf
  def pvalue(self, k = 0):
    return 1 - self.cdf(k-1)
  def estimate(self, data): #data dict value:counts
    '''estimate Poisson parameter from input data (dict type, value -> counts)
    '''
    total, cnt = data_count(data)
    self.l = float(total) / cnt
    return self.l
  def log_likelihood(self, data):
    score = 0.0
    for k in data:
      score += data[k] * self.logpmf(k)
    return score
  def r_log_like_score(self, data, l = -1):
    if l < 0 : l = self.l
    total, cnt = data_count(data)
    return float(total) / l / cnt -1
  def chisquare_test(self, data):
    total, cnt = data_count(data)
    obs, exs = [], []
    ob, ex = 0, 0
    i0 = 0
    for i in range(max(data.keys()) + 1) :
      if i in data : ob += data[i]
      ex += cnt * self.pmf(i)
      if ex >= 5 : 
        obs.append(ob)
        exs.append(ex)
        ob, ex = 0, 0
        i0 = i + 1
    ex = cnt * self.pvalue(i0)
    obs[-1] += ob
    exs[-1] += ex
    print (obs, exs, len(obs) - 1, len(exs), sum(obs), sum(exs))
    return chisquare(obs, exs)

class ZTPoisson(Poisson): 
  '''Zero truncated poisson distribution, NOT finished!
  '''
  def expect(self):
    p0 = Poisson.pmf(self, 0)
    return self.l / (1 - p0)
  def variance(self):
    p0 = Poisson.pmf(self, 0)
    return (self.l + self.l ** 2) / (1 - p0) - self.expect() ** 2
  def logpmf(self, k = 0):
    if k < 1 : return Poisson.pmf(self, -1)
    p0 = math.exp(Poisson.logpmf(self, 0))
    lp = Poisson.logpmf(self, k)
    return lp - math.log(1 - p0)
  #def pmf(self, k = 0):
    #return math.exp(self.logpmf(k))
  def cdf(self, k = 1): #, logarr = logarr):
    if k <= 0 : return 0
    #logarr_ext(k)
    lpr = self.logpmf(1)
    logl = math.log(self.l)
    cdf = math.exp(lpr)
    for i in range(2, k + 1):
      lpr += logl - math.log(i) # logarr[i]
      cdf += math.exp(lpr)
    return cdf
  #def pvalue(self, k = 0):
    #return 1 - self.cdf(k - 1)
  def estimate(self, data, max_iter = 1e4, nlike = 10): ###### NOT FINISHED!!
    total, cnt = data_count(data)
    lastllh = 0
    i = 0
    for j in range(max_iter) :
      ez = self.expected_zeros(cnt)
      data[0] = ez
      negbinom.estimate(self, data)
      data[0] = 0
      i += 1
      if i < nlike : continue
      i = 0
      llh = self.log_likelihood(data)
      d = abs(2 * (llh - lastllh) / (llh + lastllh) / nlike)
      if d < self.Delta : break
      print (d, llh, self.r, self.p, ez)
      lastllh = llh
    return self.r, self.p
  def log_likelihood(self, data):
    score = 0.0
    for k in data:
      score += data[k] * self.logpmf(k)
    return score
  def r_log_like_score(self, data, r = -1):
    if r < 0 : r = self.r
    total, cnt = data_count(data)
    #dr = scipy.special.digamma(r)
    #score = math.log(self.q) - dr
    s1, d = 0, 0
    for i in range(max(data.keys()) + 1):
      #d += 1.0 / (r + i)
      if i in data : s1 += d * data[i]
      d += 1.0 / (r + i)
    score = s1 / cnt + math.log(r / (r + 1.0*total/cnt))
    return score
  def chisquare_test(self, data):
    total, cnt = data_count(data)
    obs, exs = [], []
    ob, ex = 0, 0
    i0 = 0
    for i in range(max(data.keys()) + 1) :
      if i in data : ob += data[i]
      ex += cnt * self.pmf(i)
      if ex >= 5 : 
        obs.append(ob)
        exs.append(ex)
        ob, ex = 0, 0
        i0 = i + 1
    ex = cnt * self.pvalue(i0)
    #obs.append(ob)
    #exs.append(ex)
    obs[-1] += ob
    exs[-1] += ex
    print (obs, exs, len(obs) - 1, len(exs), sum(obs), sum(exs))
    return chisquare(obs, exs)

class rankSumTiesExact: 
  '''exact rank sum test of x < y (one-sided) for ribo
  '''
  def __init__(self, x, y, show = False):
    '''x, y are list of samples
    '''
    self.a = list(x) + list(y)
    self.x = list(x)
    self.N, self.n = len(self.a), len(x)
    self.cd = countDict(self.a) # count dict
    self.xcd = countDict(x)
    self.rd = rankDict(self.cd)
    self.xrs =  self.rankSum(self.xcd)
    self.ks = sorted(self.cd)
    self.l = len(self.ks)
    self.vs = [self.count(i) for i in range(self.l)]
    if show : 
      print (self.cd, self.xcd, self.complexity()) #print (self.tieRatio()) print (self.complexity()) #print self.numStats()
  def copy(self, other):
    self.a = other.a
    self.x = other.x
    self.N, self.n = other.N, other.n
    self.cd = other.cd # count dict
    self.xcd = other.xcd
    self.rd = other.rd
    self.xrs =  other.xrs
    self.ks = other.ks
    self.l = other.l
    self.vs = other.vs
  def complexity(self):
    '''sum of log(n+1) for each ties
    '''
    #logarr_ext(max(self.cd.values()) + 1)
    complog = sum([math.log(self.count(i)+1) for i in range(self.l)])
    return complog
  def shuffleTest(self, n = 1000, show = False):
    import random
    c = 0
    a = self.a[:]
    for i in range(n):
      random.shuffle(a)
      rcd = countDict(a[0:self.n])
      rs =  self.rankSum(rcd)
      if rs >= self.xrs: c += 1
      if show : print (a, rs, xrs, c)
      #if c >= 100 : return float(c) / (i+1) # 2 effective numbers
    #if i == 19 and c > 10 : return float(c) / 20
    #if i == 99 and c > 20 : return float(c) / 100
    return float(c) / n
  def count(self, i):
    return self.cd[self.ks[i]]
  def rank(self, i):
    return self.rd[self.ks[i]]
  def numStats(self):
    #logarr_ext(max(self.cd.values()) + 1)
    return self._numStats(self.N, self.n, 0)
  def _numStats(self, N, n, i): # NOT CORRECT!!
    n1 = N - n
    if n < 0 or n1 < 0 : return 0
    if n == 0 or n1 == 0 : return 1
    if i == self.l - 2 : return min(n, n1) + 1
    if max(self.vs[i:]) == 1 : return math.exp(combination_log(N,n))
    t1, t2 = self.count(i), N - self.count(i)
    if t1 >= n >= t2 or t1 >= n1 >= t2 : 
      d = math.exp(sum([math.log(self.count(k)+1) for k in range(i+1, self.l)]))
      return d
      #print 'multi all', d, N, n, i
    s = 0
    for ni in range(max(0, n-t1),min(self.count(i), n) + 1) :
      d = self._numStats(N - self.count(i), n - ni, i+1)
      s += d
      #print d,s,N - ni, n - ni, i+1
    return s
  def numStatsRaw(self): # use complexity() instead
    s = 1
    for i in range(self.l) :
      s *= self.count(i) + 1
    return s
  def tieRatio(self):
    '''tie correction ratio
    '''
    s = sum([v**3 - v for v in self.cd.values() if v > 1])
    return s / float(self.N ** 3 - self.N)
  def test(self, th = 10, delta = 1e-4):
    '''if either size <= th, use fastTest
    elif complexity <= th, use fastTest
    else use normal mwtest
    '''
    if self.n <= th or self.N - self.n <= th : return self.fastTest(delta = delta) #, True
    if self.complexity() <= th: return self.fastTest(delta = delta) # * logarr[2]
    return self.mwtest() #, False
  def mwtest(self, use_continuity = True, show = False):
    '''normal mwtest, alt = greater, one-tailed
    '''
    from scipy.stats import norm
    n1, n2, n = self.n, self.N - self.n, self.N
    mu = n1 * (n + 1) / 2.0
    s = sum([v**3 - v for v in self.cd.values() if v > 1])
    if show : print ('effect size {}'.format(n - s / float(n*(n-1))))
    var = n1 * n2 * (n ** 3 - n - s) / 12.0 / n / (n - 1)
    if use_continuity : z = (self.xrs - mu - 0.5) / var ** 0.5
    else : z = (self.xrs - mu) / var ** 0.5
    p = norm.sf(z)
    if show : print (self.xrs, mu, self.xrs-mu+n1*(n1+1)/2.0, var, z, p)
    return p
    #else : return 1 - p # one sided
  def isExtreme(self, rs, twotailed = False, alt = 'g', delta = 1e-5): # not used. if the given rank sum is farther than xrs
    if twotailed : 
      if not hasattr(self, 'mu'):
        n1, n2, n = self.n, self.N - self.n, self.N
        self.mu = n1 * (n + 1) / 2.0
        self.th = abs(self.xrs - self.mu)
      return abs(prs - self.mu) >= self.th - delta
    elif alt == 'g' : return rs >= self.xrs - delta
    else : return rs <= self.xrs + delta
  def exactTest(self, twotailed = False, show = False): 
    '''get exact p-value for ranksum permutation test, time consuming.
    '''
    pval = 0
    if twotailed : 
      n1, n2, n = self.n, self.N - self.n, self.N
      mu = n1 * (n + 1) / 2.0
      th = abs(self.xrs - mu)
    for pcd in self.multiHypergeoIter(self.n, (), 0, self.N):
      prs =  self.rankSum(pcd)
      #if show : print prs, pcd
      if twotailed : 
        if abs(prs - mu) < th - 0.0001 : continue
      elif prs < self.xrs - 0.0001 : continue
      p = multiHypergeoProb(pcd, self.cd)
      pval += p
      if show : print (prs, pcd, p)
    if pval > 1.0 : pval = 1.0
    return pval
  def fastTest(self, show = False, delta = 1e-4):
    '''fast version of exactTest, reduced time at some cost of accuracy.
    '''
    self.pval = 0
    self.lp0 = - combination_log(self.N, self.n)
    for pcd in self.multiHypergeoFastIter(self.n, (), 0, self.N, 0, delta=delta):
      p = multiHypergeoMergeProb(pcd, self.cd, self.N, self.n, lp0 = self.lp0) # merged probability
      self.pval += p
      if show : print (p, pcd)
    if self.pval > 1.0 : self.pval = 1.0
    return self.pval
  def multiHypergeoIter(self, n, vs, i, N):
    if n == 0 : yield zipDict(self.ks, vs) # no more, return
    elif n > N : return # impossible
    elif i + 1 == len(self.ks) : # the last key
      if n <= self.cd[self.ks[i]] : yield zipDict(self.ks, vs + (n,))
      return
    else :
      N1 = N - self.count(i)
      for j in range(max(0, n-N1), min(n, self.count(i)) + 1):
        for pcd in self.multiHypergeoIter(n-j, vs+(j,), i+1, N1):
          yield pcd
  def multiHypergeoFastIter(self, n, vs, i, N, rs, delta=1e-4): 
    '''fast iter, only select rank sum higher than x rank sum conditions
    '''
    if n > N : return # impossible
    elif n == 0 : yield zipDict(self.ks, vs) # no more, return
    elif i + 1 == len(self.ks) : # the last key
      if n <= self.count(i) : yield zipDict(self.ks, vs + (n,))
      return
    else :
      N1 = N - self.count(i)
      jd, ju = max(0, n-N1), min(n, self.count(i)) + 1
      rsu, rsd = {}, {}
      j1, j2 = jd, ju
      while j1 < j2 : # looking for threshold of all lower conditions
        j = (j1 + j2) // 2 # + 1 for upper int
        rsu[j] = self.rankSumUpLimit(vs+(j,), rs+j*self.rank(i))
        if rsu[j] < self.xrs - 0.0001 : j2 = j # only keep possible conditions
        else : j1 = j + 1
      ju = j1
      j1, j2 = jd, ju
      while j1 < j2 : # looking for threshold of all higher conditions
        j = (j1 + j2) // 2 # + 1 for upper int
        rsd[j] = self.rankSumDownLimit(vs+(j,), rs+j*self.rank(i))
        if rsd[j] >= self.xrs - 0.0001 : j1 = j + 1 # only keep partial possible conditions
        else : j2 = j
      for j in range(jd, j1) : yield zipDict(self.ks, vs+(j,))
      ps = [multiHypergeoMergeProb(zipDict(self.ks, vs+(j,)), self.cd, self.N, self.n, lp0=self.lp0) for j in range(j1, ju)]
      for jo in orderIter(ps, reverse = True): #range(j1, ju):
        j = j1 + jo
        vsj, rsj = vs+(j,), rs+j*self.rank(i)
        if self.pval > 0 and ps[jo] / self.pval < delta : # whether prob. high enough to calculate in detail
          self.pval += ps[jo] / 2 # 0 order approximation; ps[jo] * ratio # 1st order approximation
        else :
          for pcd in self.multiHypergeoFastIter(n-j, vsj, i+1, N1, rsj, delta=delta):
            yield pcd
  def rankSum(self, cd):
    return sum([self.rd[k] * cd[k] for k in cd])
  def rankSumUpLimit(self, vs, rs): # rank sum up & down limit
    n, i = self.n - sum(vs), len(self.ks) - 1
    while n >= 1 :
      if n > self.count(i) : d = self.count(i)
      else : d = n
      rs += d * self.rank(i)
      n -= d
      i -= 1
    return rs
  def rankSumDownLimit(self, vs, rs): # rank sum up & down limit
    n, i = self.n - sum(vs), len(vs)
    while n >= 1 : 
      #print n, i
      if n > self.count(i) : d = self.count(i)
      else : d = n
      rs += d * self.rank(i)
      n -= d
      i += 1
    return rs
def countDict(arr):
  '''count array to dict
  '''
  data = {}
  for d in arr:
    if d not in data : data[d] = 0
    data[d] += 1
  return data
def rankDict(data):
  '''count array to rank dict
  '''
  ks = sorted(data)
  total = 0
  rd = {}
  for k in ks:
    total += data[k]
    rd[k] = total - (data[k] - 1) / 2.0
  return rd

def zipDict(ks, vs):
  data = {}
  for i, v in enumerate(vs) : data[ks[i]] = v
  return data

def orderIter(arr, reverse = False):
  '''generate order of given array. e.g. the index of smallest item come first
  '''
  ad = {}
  for i, v in enumerate(arr): 
    if v not in ad : ad[v] = []
    ad[v].append(i)
  a = sorted(ad, reverse = reverse)
  for v in a : 
    for i in ad[v] : yield i

def order(arr, reverse = False):
  return [i for i in orderIter(arr, reverse = reverse)]

def multiHypergeoProb(pcd, cd, show = False):
  '''pmf of multi-hypergeomic distribution, cd provides total numbers, pcd provides selected numbers
  '''
  lp = - combination_log(sum(cd.values()), sum(pcd.values()))
  if show : print (sum(cd.values()), sum(pcd.values()), lp)
  for k in pcd :
    lp += combination_log(cd[k], pcd[k])
    if show : print (cd[k], pcd[k], lp)
  return math.exp(lp)
def multiHypergeoMergeProb(pcd, cd, N, n, lp0 = None, show = False):
  '''keys not in pcd/cd are not considered. total numbers are provided by N & n
  '''
  if lp0 is None : lp = - combination_log(N, n)
  else : lp = lp0
  if show : print (lp)
  for k in pcd :
    lp += combination_log(cd[k], pcd[k])
    n -= pcd[k]
    N -= cd[k]
    if show : print (cd[k], pcd[k], N, n, lp)
  lp += combination_log(N, n)
  return math.exp(lp)

def glmNBTest(x, y):
  '''test difference of two array of counts by generized linear model with negative binomial family
  require statsmodels module
  '''
  import statsmodels.formula.api as smf
  import statsmodels.api as sm
  if max(x) == 0 : return 1.
  data = {}
  data['data'] = list(x) + list(y)
  data['grp'] = [1]*len(x) + [0]*len(y)
  model=smf.glm("data ~ grp", data=data, family=sm.families.NegativeBinomial()).fit()
  p = model.pvalues[1] / 2
  if model.tvalues[1] >= 0 : return p
  else : return 1-p

class betaBinom:
  ''' Beta-Binomial distribution model
  '''
  def __init__(self, a, b): # alpha & beta
    self.a = a
    self.b = b
  def logpmf(self, n, k):
    return combination_log(n, k) + betaln(k + self.a, n - k + self.b) - betaln(self.a, self.b)
  def pmf(self, n, k):
    return math.exp(self.logpmf(n, k))
  def cdf(self, n, k):
    p = 0
    for i in range(k+1):
      p += self.pmf(n, i)
    if p > 1 : p = 1
    return p
  def pvalue(self, n, k, alt = 'two.sided'):
    cdf = self.cdf(n, k)
    if alt in ('l', 'less') : return cdf
    else :
      p = 1 - cdf + self.pmf(n, k)
      if alt in ('g', 'greater') : return p
      else :
        m = min(cdf, p) * 2
        if m > 1 : m = 1
        return m
    #if alt not in ('g', 'greater') : return cdf
    #else :
      #p = self.pmf(n, k)
      #return 1 - cdf + p




from ribotish.zbio import gtf, bam, ribo, stat, exp, tools, orf, fa, interval, io
import math, time, itertools
from os.path import isfile

def help():
  return "Predict ORFs from riboseq data"
def set_parser(parser):
  #### basic options ####
  parser.add_argument("-t", type=strlist, dest="tisbampaths", default=[], help="TIS enriched riboseq bam files, comma seperated")
  parser.add_argument("-b", type=strlist, dest="ribobampaths", default=[], help="Ordinary riboseq bam files, comma seperated")
  parser.add_argument("-g", type=str, dest="genepath", required=True, help='Gene annotation file for ORF prediction')
  parser.add_argument("-f", type=str, dest="genomefapath", required=True, help="Genome fasta file")
  parser.add_argument("-o", type=str, dest="output", default='predict.txt', required=True, help="Output result file")
  #### alt input options ####
  parser.add_argument("-i", type=str, dest="input", help="Only test input candidate ORFs, format: transID start stop (0 based, half open)")
  parser.add_argument("--geneformat", type=str, default='auto', help="Gene annotation file format (gtf, bed, gpd, gff, default: auto)")
  parser.add_argument("--tispara", type=strlist, help="Input offset parameter files for -t bam files")
  parser.add_argument("--ribopara", type=strlist, help="Input offset parameter files for -b bam files")
  parser.add_argument("--nparts", type=int, default=10, help="Group transcript according to TIS reads density quantile (default: 10)")
  parser.add_argument("-a", type=str, dest="agenepath", help="Gene file for known protein coding gene annotation and TIS background estimation instead of -g gene file")
  parser.add_argument("-e", type=str, dest="estpath", default='tisBackground.txt', help="Output TIS background estimation result (default: tisBackground.txt)")
  parser.add_argument("-s", type=str, dest="inestpath", help="Input background estimation result file instead of instant estimation")
  parser.add_argument("--transprofile", type=str, help="Output RPF P-site profile for each transcript")
  parser.add_argument("--inprofile", type=str, help="Input RPF P-site profile for each transcript, instead of reading bam reads, save time for re-running")
  parser.add_argument("--chrmap", type=str, help="Input chromosome id mapping table file if annotation chr ids are not same as chr ids in bam/fasta files")
  parser.add_argument("--allresult", type=str, help="All result output without FDR q-value threshold (default: output + '_all.txt', 'off' to turn off)")
  # predict options
  parser.add_argument("--alt", action="store_true", help="Use alternative start codons (all codons with 1 base different from ATG)")
  parser.add_argument("--altcodons", type=strlist, help="Use provided alternative start codons, comma seperated, eg. CTG,GTG,ACG")
  parser.add_argument("--tis2ribo", action="store_true", help="Add TIS bam counts to ribo, if specified or -b not provided")
  parser.add_argument("--harr", action="store_true", help="The data is treated with harringtonine (instead of LTM)")
  parser.add_argument("--harrwidth", type=int, default=15, help="Flanking region for harr data, in codons (default: 15)")
  parser.add_argument("--enrichtest", action="store_true", help="Use enrich test instead of frame test")
  # output options
  parser.add_argument("--minaalen", type=int, default=6, help="Min amino acid length of candidate ORF (default: 6)")
  parser.add_argument("--genefilter", type=strlist, help="Only process given genes")
  parser.add_argument("--tpth", type=float, default=0.05, help="TIS p value threshold (default: 0.05)")
  parser.add_argument("--fpth", type=float, default=0.05, help="Frame p value threshold (default: 0.05)")
  parser.add_argument("--minpth", type=float, default=1, help="At least one of TIS or frame p value should be lower than this threshold (default: 0.05)")
  parser.add_argument("--fspth", type=float, default=0.05, help="Fisher's p value threshold")
  parser.add_argument("--fsqth", type=float, default=0.05, help="Fisher's FDR q value threshold")
  parser.add_argument("--framelocalbest", action="store_true", help="Only report local best frame test results")
  parser.add_argument("--framebest", action="store_true", help="Only report best frame test results")
  parser.add_argument("--longest", action="store_true", help="Only report longest possible ORF results")
  parser.add_argument("--seq", action="store_true", help="Report ORF sequences")
  parser.add_argument("--aaseq", action="store_true", help="Report amino acid sequences")
  parser.add_argument("--blocks", action="store_true", help="Report all exon block positions for predicted ORFs")
  parser.add_argument("--inframecount", action="store_true", help="Report the sum of all counts at the in-frame positions in the ORF")
  # Reads filters
  parser.add_argument("--maxNH", type=int, default=5, help="Max NH value allowed for bam alignments (default: 5)")
  parser.add_argument("--minMapQ", type=float, default=1, help="Min MapQ value required for bam alignments (default: 1)")
  parser.add_argument("--secondary", action="store_true", help="Use bam secondary alignments")
  parser.add_argument("--paired", action="store_true", help="Reads are paired end")
  parser.add_argument("--nocompatible", action="store_true", help="Do not require reads compatible with transcript splice junctions")
  parser.add_argument("--compatiblemis", type=int, default=2, help="Missed bases allowed in reads compatibility check (default: 2)")
  # running options
  parser.add_argument("-p", type=int, dest="numProc", default=1, help="Number of processes")
  parser.add_argument("-v", "--verbose", action="count", default=0, help="Increase output verbosity")

def strlist(s):
  '''Convert comma seperated file name string to list
  '''
  return s.split(',')

try: imap = itertools.imap
except: imap = map

def run(args):
  '''Main function for ORF finding
  '''
  # prepare
  global tisbampaths, tisoffdict, ribobampaths, riboffdict, genomefapath, compatible, compatiblemis
  global minaalen, enrichtest, slp, paras, verbose, alt, title, tis2ribo, gfilter
  global tpth, fpth, minpth, fspth, framebest, framelocalbest, longest, transprofile, TIS_types #fspth
  global paired, seq, aaseq, blocks, inframecount # showtime
  paired, seq, aaseq, blocks = args.paired, args.seq, args.aaseq, args.blocks
  inframecount = args.inframecount
  ribo.maxNH, ribo.minMapQ, ribo.secondary = args.maxNH, args.minMapQ, args.secondary
  tisbampaths = args.tisbampaths
  ribobampaths = args.ribobampaths
  if len(tisbampaths) == 0 and len(ribobampaths) == 0 :
    print('No bam file input!')
    exit(1)
  genomefapath = args.genomefapath
  compatible = not args.nocompatible
  compatiblemis = args.compatiblemis
  minaalen = args.minaalen
  enrichtest = args.enrichtest
  transprofile = args.transprofile
  harrwidth = None
  TIS_types = ['Annotated', 'Truncated', 'Extended', "5'UTR", "3'UTR", 'Internal', 'Novel']
  if args.chrmap is not None :
    chrmap = {}
    for lst in io.splitIter(args.chrmap, sep=None):
      if len(lst) < 2: continue
      chrmap[lst[0]] = lst[1]
      chrmap[lst[1]] = lst[0]
    bam.chrmap = chrmap
    fa.chrmap = chrmap
  if args.harrwidth is not None : harrwidth = args.harrwidth
  elif args.harr : harrwidth = 15
  verbose = args.verbose
  alt = args.alt
  if args.altcodons is not None : 
    alt = True
    if args.altcodons[0].upper() == 'ALL' : orf.cstartlike = orf.allcodons
    else : orf.cstartlike = [c.upper() for c in args.altcodons]
  tpth, fpth, minpth, framebest, framelocalbest = args.tpth, args.fpth, args.minpth, args.framebest, args.framelocalbest # fspth
  fspth = args.fspth
  longest = args.longest
  tis2ribo = args.tis2ribo
  parts = [0.1 * (i+1) for i in range(args.nparts)]
  gfilter = None
  if args.genefilter is not None :
    gfilter = {}
    for gid in args.genefilter : gfilter[gid] = 1
  flank = 3 ##
  tisoffdict = find_offset(args.tisbampaths, args.tispara)
  riboffdict = find_offset(args.ribobampaths, args.ribopara)
  if len(args.ribobampaths) == 0 : 
    print('No regular RiboSeq data input. TIS data will also be used as regular RiboSeq data.')
    tis2ribo = True
  if len(args.tisbampaths) == 1 : 
    if args.inestpath is None : 
      path = args.tisbampaths[0] + '.bgest.txt'
      if isfile(path) : args.inestpath = path
      elif args.estpath is None or args.estpath == '' : args.estpath = path
  if args.agenepath is None : args.agenepath = args.genepath

  # load genome, fasta file indexing
  if args.verbose : print("{} Loading genome...".format(time.ctime()))
  genome = fa.Fa(args.genomefapath, verbose = args.verbose)

  # TIS background estimation
  if len(args.tisbampaths) == 0 : 
    print('No input TIS data!')
    paras, slp = [(1,0.5)], [1] # No TIS input
  elif args.inestpath is None : #== '' :
    print ("{} Estimating TIS background parameters...".format(time.ctime()))
    if args.verbose : print("TIS background estimation result will be saved to {}".format(args.estpath))
    if args.numProc > 1 : 
      from multiprocessing import Process
      import multiprocessing.pool
      class NoDaemonProcess(Process):
      # make 'daemon' attribute always return False
        def _get_daemon(self):
          return False
        def _set_daemon(self, value):
          pass
        daemon = property(_get_daemon, _set_daemon)
      class MyPool(multiprocessing.pool.Pool):
        Process = NoDaemonProcess

      pool = MyPool(1) # This is for memory efficiency
      paras, slp, data = pool.apply(ribo.estimateTISbg, args=(args.agenepath, args.tisbampaths, args.genomefapath), kwds={'parts': parts, 'offdict': tisoffdict, 'alt_tis': alt, 'numProc': args.numProc, 'verbose': args.verbose, 'geneformat': args.geneformat, 'harrwidth': harrwidth, 'paired': paired})
      pool.close()
    else : 
      paras, slp, data = ribo.estimateTISbg(args.genepath, args.tisbampaths, args.genomefapath, parts = parts, offdict = tisoffdict, alt_tis=alt, numProc = 1, verbose = verbose, geneformat = args.geneformat, harrwidth = harrwidth, paired = paired)
    estfile = open(args.estpath, 'w')
    for i in range(len(parts)):
      estfile.write("{}\t{}\t{}\t{}\t{}\n".format(paras[i][0], paras[i][1], parts[i], slp[i], data[i]))
    estfile.close()
    
  else : 
    print("Loading TIS background file {}...".format(args.inestpath))
    inestfile = open(args.inestpath, 'r')
    paras, slp = [], []
    for l in inestfile:
      lst = l.strip().split('\t')
      paras.append((float(lst[0]), float(lst[1])))
      slp.append(eval(lst[3]))

  if args.inprofile is not None and not isfile(args.inprofile):
    print('inprofile {} not found!'.format(args.inprofile))
    if args.transprofile is None:
      transprofile = args.inprofile

  if args.numProc > 1 : 
    from multiprocessing import Pool
    pool = Pool(processes = args.numProc - 1) 

  cds_regions = {}
  known_tis = {}
  if args.agenepath != args.genepath :
    if verbose : print('Loading CDS annotation...')
    for g in io.geneIter(args.agenepath, fileType = args.geneformat, verbose = args.verbose):
      if genome.get_chrname(g.chr) is None: continue
      if g.chr not in cds_regions :
        cds_regions[g.chr] = {'+':[interval.Interval() for i in range(3)], '-':[interval.Interval() for i in range(3)]}
        known_tis[g.chr] = {'+':{}, '-':{}}
      for t in g.trans:
        cr = interval.cds_region_trans(t)
        for i in range(3) :
          cds_regions[t.chr][t.strand][i].lst += cr[i].lst
      #for t in g.trans :
        tis = t.cds_start(cdna = False)
        if tis is not None : known_tis[t.chr][t.strand][tis] = 1

  inorf = None
  if args.input is not None :
    if verbose : print('Loading candidates...')
    inorf = {}
    infile = open(args.input, 'r')
    for l in infile :
      lst = l.strip().split()
      tid, tis, stop = lst[0], int(lst[1]), int(lst[2])
      #if gfilter is not None and tid not in gfilter : continue
      if tid not in inorf : inorf[tid] = []
      inorf[tid].append([tis, stop])
  inprofile = None
  if args.inprofile is not None :
    if isfile(args.inprofile):
      if verbose : print('Loading transcript profile...')
      inprofile = {}
      for lst in io.splitIter(args.inprofile):
        try : gid, tid, tispf, ribopf = lst[0], lst[1], eval(lst[3]), eval(lst[4])
        except : continue
        if gid not in inprofile : inprofile[gid] = {}
        inprofile[gid][tid] = tispf, ribopf

  print("{} Predicting...".format(time.ctime()))
  profile = exp.Profile()
  title = ['TISGroup', 'TISCounts', 'TISPvalue', 'RiboPvalue', 'RiboPStatus']
  j = [0,0] # total number of ORF/TIS for BH correction
  gene_iter = io.geneIter(args.genepath, fileType = args.geneformat, verbose = args.verbose) # chrs = genome.idx
  para_iter = genePara(gene_iter, inorf, inprofile)
  if args.numProc <= 1 : pred_iter = imap(_pred_gene, para_iter)
  else : 
    #from multiprocessing import Pool
    #pool = Pool(processes = args.numProc - 1)
    pred_iter = pool.imap_unordered(_pred_gene, para_iter, chunksize = 5)
  if transprofile is not None : 
    tpfile = open(transprofile, 'w')
    tpfile.write('Gid\tTid\tSymbol\tTISProf\tRiboProf\n')

  for result in pred_iter:
    es, ji, tpfs, g = result
    j[0] += ji[0]
    j[1] += ji[1]
    for e in es : 
      profile.add_exp(e)
      if verbose >= 2 : print('{} {}'.format(time.ctime(), str(e)))
    if transprofile is not None : 
      for tid in tpfs :
        tpfile.write(io.tabjoin(tid, tpfs[tid])+'\n')
    if g.chr not in cds_regions : 
      cds_regions[g.chr] = {'+':[interval.Interval() for i in range(3)], '-':[interval.Interval() for i in range(3)]}
      known_tis[g.chr] = {'+':{}, '-':{}}
    for t in g.trans:
      cr = interval.cds_region_trans(t)
      for i in range(3) :
        cds_regions[t.chr][t.strand][i].lst += cr[i].lst
    #for t in g.trans : 
      tis = t.cds_start(cdna = False)
      if tis is not None : known_tis[t.chr][t.strand][tis] = 1

  for chr in cds_regions :
    for strand in cds_regions[chr] :
      for i in range(3) :
        cds_regions[chr][strand][i].check()
  print("{} Checking overlap with known CDS..".format(time.ctime()))
  for e in profile : 
    if e.tistype == 0 : continue
    elif e.gtis in known_tis[e.chr][e.strand] : e.id += ':Known'
    elif e.tistype > 1 : # ["5'UTR", "3'UTR", "Inside", "Novel", 'Extended'] :
      #coding_overlap = False
      for i in range(3) :
        its = cds_regions[e.chr][e.strand][i].intersect(e.cr[i]) # e.cr[i].intersect(cds_regions[e.chr][e.strand][i])
        if its.rlen() > 0 : 
          #coding_overlap = True
          e.id += ':CDSFrameOverlap'
          break

  print("{} BH correcting...".format(time.ctime()))
  profile.BHcorrection(2, total = j[1], append = True) # Calculate BH FDR of TIS p value
  profile.BHcorrection(3, total = j[0], append = True) # Frame p value
  i = 1
  if len(tisbampaths) == 0 : i = 0
  profile.BHcorrection(5, total = j[i], append = True) # Calculate BH FDR for Fisher's p value

  outfile = open(args.output,'w')
  s = "Gid\tTid\tSymbol\tGeneType\tGenomePos\tStartCodon\tStart\tStop\tTisType\t"
  s += '\t'.join(title)
  s += '\tFisherPvalue\tTISQvalue\tFrameQvalue\tFisherQvalue\tAALen'
  if seq : s += '\tSeq'
  if aaseq : s += '\tAASeq'
  if blocks: s += '\tBlocks'
  if inframecount: s += '\tInFrameCount'
  s += '\n'
  outfile.write(s)

  if args.allresult is not None and args.allresult.upper() == 'OFF' : allout = None
  elif args.fsqth == 1: allout = None
  else : 
    if args.allresult is None : 
      lst = args.output.split('.')
      if lst[-1] == 'txt' : args.allresult = args.output[:-4] + '_all.txt' 
      else : args.allresult = args.output + '_all.txt'
    allout = open(args.allresult, 'w')
    allout.write(s)

  for e in profile:
    #if e.q > args.fsqth : continue
    if len(tisbampaths) == 0 : e.data[5], e.data[8] = None, None # No Fisher's
    s = "%s\t%d" % (e, e.length)
    if seq : s += '\t' + e.sq
    if aaseq : s += '\t' + e.aa
    if blocks: s += '\t' + e.blocks
    if inframecount: s += '\t{}'.format(e.inframecount)
    s += '\n'
    if allout is not None : allout.write(s)
    if e.q <= args.fsqth : outfile.write(s) # "%s\t%d\n" % (e, e.length)) #, e.sq))
  
def check_overlap(e, known_tis, cds_regions) : 
  if e.tistype == 0 : return ''
  elif e.gtis in known_tis : return ':Known'
  elif e.tistype > 1 : # ["5'UTR", "3'UTR", "Inside", "Novel", 'Extended'] :
    for i in range(3) :
      its = e.cr[i].intersect(cds_regions[i])
      if its.rlen() > 0 : return ':CDSFrameOverlap'

def genePara(gene_iter, inorf, inprofile):
  '''Generate parameters (gene, candidates/None, profile) for function _pred_gene()
  '''
  if inorf is not None :
    for g in gene_iter:
      pf = {}
      if inprofile is not None and g.id in inprofile : pf = inprofile[g.id]
      cand = {}
      for t in g.trans:
        if t.id in inorf : cand[t.id] = inorf[t.id] #yield t, inorf[t.id]
      #if len(cand) > 0 : 
      yield g, cand, pf
  else :
    for g in gene_iter: 
      if gfilter is not None :
        if g.id not in gfilter : continue # or t.gid not in gfilter : continue
      pf = {}
      if inprofile is not None and g.id in inprofile : pf = inprofile[g.id]
      yield g, None, pf

def find_offset(bampaths, para):
  '''Get offset data for given bam data
  '''
  offlist = [None] * len(bampaths)
  for i, path in enumerate(bampaths):
    if para is None or i > len(para)-1 or para[i] == '' : path = path + '.para.py'
    else : path = para[i]
    if isfile(path) : 
      vessel = {}#'offdict': None}
      exec(open(path).read(), vessel) # execfile(path, vessel)
      offlist[i] = vessel['offdict']
  for i, od in enumerate(offlist):
    if od is None :
      print('No offset parameter file found for %s. Using default offset (12). ' %  bampaths[i])
  return offlist
    
def _pred_gene(ps): ### trans
  '''Main function of ORF prediction in given transcript
  '''
  g, candidates, pf = ps
  es, j = [], [0,0]
  tpfs = {} #trans profiles
  genome = fa.Fa(genomefapath)
  has_tis = len(tisbampaths) > 0
  if genome.get_chrname(g.chr) is None: return es, j, tpfs, g

  load = True
  if candidates is not None :
    if len(candidates) == 0 :  return es, j, tpfs, g
    if len(pf) >= len(candidates) : load = False
  if len(pf) >= len(g.trans) : load = False
  if load :
    tismbl = ribo.multiRiboGene(g, tisbampaths, offdict = tisoffdict, compatible = compatible, mis = compatiblemis, paired = paired)
    ribombl = ribo.multiRiboGene(g, ribobampaths, offdict = riboffdict, compatible = compatible, mis = compatiblemis, paired = paired)

  for t in g.trans:
    #if candidates is not None and t.id not in candidates : continue
    tl = t.cdna_length()
    if tl < ribo.minTransLen : continue # return es, j, tpfs, g ##
    #ttis = ribo.multiRibo(t, tisbampaths, offdict = tisoffdict, compatible = compatible)
    #tribo = ribo.multiRibo(t, ribobampaths, offdict = riboffdict, compatible = compatible)
    if not load :
      if t.id in pf : 
        tispf, ribopf = pf[t.id]
        ttis = ribo.Ribo(t)
        tribo = ribo.Ribo(t)
        if has_tis : ttis.dict2cnts(tispf)
        tribo.dict2cnts(ribopf)
      else : 
        print('Warning: transcript {} {} {} not in input trans profile! '.format(t.gid, t.id, t.symbol))
        continue
    else :
      ttis = ribo.Ribo(t, bamload = tismbl, compatible = compatible, mis = compatiblemis)
      tribo = ribo.Ribo(t, bamload = ribombl, compatible = compatible, mis = compatiblemis)

    score = ttis.abdscore()
    if score is not None: ip = ribo.pidx(score, slp)   
    else: ip = 0
    if verbose >= 2 : print(io.tabjoin(g.id, t.id, ttis.total, tribo.total))
    cds1 = t.cds_start(cdna = True) 
    cds2 = t.cds_stop(cdna = True) 
    tsq = genome.transSeq(t)

    if transprofile is not None:
      tid = '{}\t{}\t{}'.format(t.gid, t.id, t.symbol)
      tpfs[tid] = '{}\t{}'.format(ttis.cnts_dict_str(), tribo.cnts_dict_str())

    if has_tis and tis2ribo : tribo.merge(ttis) ##

  # user provided candidates
    if candidates is not None : 
      if t.id not in candidates : continue
      for tis, stop in candidates[t.id]:
        j[0] += 1
        j[1] += 1
        if has_tis : tp = ttis.tis_test(tis, paras[ip][0], paras[ip][1])
        else : tp = None
        if enrichtest : rp = tribo.enrich_test(tis, stop)
        else : rp = tribo.frame_test(tis, stop)
        if tp is not None and tp > tpth : continue 
        if rp > fpth : continue # or fisher > fspth 
        minp = rp
        if tp is not None and tp < minp : minp = tp
        if minp > minpth : continue
        fsp, fss = stat.fisher_method([tp, rp]) #
        if tp is not None and fsp > fspth : continue
        has_stop = tsq[stop-3:stop] in orf.cstop
        e = getResult(t, tis, stop, cds1, cds2, tsq, [ip, ttis.cnts[tis], tp, rp, 'N', fsp], has_stop)
        if inframecount: e.inframecount = sum(tribo.cnts[tis:stop:3])
        es.append(e)

    else : #all possible ORFs
      orfs = orf.orflist(tsq, minaalen = minaalen, tail = tl)
      for o in orfs :
        starts = o.starts
        if alt : starts += o.altstarts
        starts.sort()
        if longest and not has_tis : starts = starts[0:1]
        ol = len(starts)
        if ol == 0 : continue
        tps = [None] * ol
        rps = [1] * ol
        if has_tis : allz_tis = max(ttis.cnts[starts[0]:o.stop:3]) == 0 # all zeros
        else : allz_tis = True
        allz_ribo = max(tribo.cnts[starts[0]:o.stop:3]) == 0 # all zeros
        if allz_tis and allz_ribo : continue
        for i, tis in enumerate(starts) : 
          if has_tis : tps[i] = ttis.tis_test(tis, paras[ip][0], paras[ip][1])
          if not allz_ribo : allz_ribo = max(tribo.cnts[tis:o.stop:3]) == 0
          if not allz_ribo :
            if enrichtest : rps[i] = tribo.enrich_test(tis, o.stop)
            else : rps[i] = tribo.frame_test(tis, o.stop)
        rst = pvalStatus(rps)
        for i, tis in enumerate(starts) : 
          if tps[i] is not None and tps[i] > tpth : continue
          if rps[i] > fpth : continue # or fishers[i] > fspth
          minp = rps[i]
          if tps[i] is not None and tps[i] < minp : minp = tps[i]
          if minp > minpth : continue
          if tps[i] is None or tps[i] > minpth :
            if longest : 
              if i > 0 : continue
            else :
              if framelocalbest and rst[i] == 'N' : continue
              if framebest and rst[i][0] != 'T' : continue
          fsp, fss = stat.fisher_method([tps[i], rps[i]]) #
          if fsp > fspth : continue
          e = getResult(t, tis, o.stop, cds1, cds2, tsq, [ip, ttis.cnts[tis], tps[i], rps[i], rst[i], fsp], o.has_stop_codon)
          if inframecount: e.inframecount = sum(tribo.cnts[tis:o.stop:3])
          #tistype = tisType(tis, o.stop, cds1, cds2)
          #orfstr = '{}\t{}\t{}'.format(tsq[tis:tis+3],tis,o.stop)
          #tid = "%s\t%s\t%s\t%s\t%s:%d-%d:%s\t%s\t%s" % (t.gid, t.id, t.symbol, t.genetype, t.chr, t.genome_pos(tis), t.genome_pos(o.stop), t.strand, orfstr, tistype)
          #values = [ip, ttis.cnts[tis], tps[i], rps[i], rst[i]] # , fishers[i]]
          #e = exp.Exp(tid, values)
          #e.length = (o.stop - tis) / 3 - 1
          #e.sq = tsq[tis:o.stop]
          #e.chr, e.strand, e.tistype = t.chr, t.strand, tistype
          #if e.tistype == 'Extended' : e.cr = interval.cds_region_trans(t, tis, tis+3)
          #else : e.cr = interval.cds_region_trans(t, tis, o.stop)
          es.append(e)
        #if has_tis : 
        j[1] += ol
        j[0] += 1

  return es, j, tpfs, g

def getResult(t, tis, stop, cds1, cds2, tsq, values, has_stop = True):
  tistype = tisType(tis, stop, cds1, cds2)
  orfstr = '{}\t{}\t{}'.format(tsq[tis:tis+3], tis, stop)
  gtis, gstop = t.genome_pos(tis, bias=1), t.genome_pos(stop, bias=0)
  if t.strand != '-' : genomestr = '{}:{}-{}:{}'.format(t.chr, gtis, gstop, t.strand)
  else : genomestr = '{}:{}-{}:{}'.format(t.chr, gstop, gtis, t.strand)
  tid = "%s\t%s\t%s\t%s\t%s\t%s\t%s" % (t.gid, t.id, t.symbol, t.genetype, genomestr, orfstr, TIS_types[tistype])
  e = exp.Exp(tid, values)
  e.length = (stop - tis) / 3 - 1
  if not has_stop : e.length += 1
  if seq : e.sq = tsq[tis:stop]
  if aaseq : e.aa = orf.translate(e.sq)
  if blocks: 
    oi = interval.trans2interval(t, tis, stop)
    e.blocks = ','.join(['{}-{}'.format(r[0],r[1]) for r in oi.lst]) 
  e.chr, e.strand, e.tistype = t.chr, t.strand, tistype
  e.gtis, e.gstop = gtis, gstop
  if tistype == 2 : e.cr = interval.cds_region_trans(t, tis, tis+3) # Extended TIS region
  elif tistype > 2 : e.cr = interval.cds_region_trans(t, tis, stop) # new ORFs
  return e

def pvalStatus(ps) : 
  '''Find global best (T) & local best (L) TISs that correspond to same stop codon, suggesting potential TISs by regular RiboSeq data
  '''
  l = len(ps)
  st = ['N'] * l
  if l <= 0 : return st
  if longest and len(tisbampaths) == 0 : return st
  cp = ps[:] + [1]
  m, c = 1, 1
  for i, p in enumerate(cp): 
    if p is None : cp[i] = 1
    elif p < m : m, c = p, 1
    elif p == m : c += 1
  if m >= 1 : return st
  top = 'T'
  if c > 1 : top = 'T{}'.format(c)
  last, c = 1, 1
  better = True
  for i, p in enumerate(cp): 
    if p == m : st[i] = top
    elif p < last : 
      c = 1
      better = True
    elif p == last : c += 1
    else : 
      if better and last > m : 
        local = 'L'
        if c > 1 : local = 'L{}'.format(c)
        for j in range(i-c, i) : st[j] = local
      better = False
      c = 1
    last = p
  return st

def tisType(start, stop, cds1, cds2):
  '''Type of the TIS relatve to annotated ORF
  '''
  if cds1 is None : return 6 # 'Novel'
  else :
    if start == cds1 : return 0 # 'Annotated'
    elif cds1 < stop <= cds2 and (stop - cds2) % 3 == 0 : # in frame
      if start > cds1 : return 1 # 'Truncated'
      else : return 2 # 'Extended'
    else : 
      if start < cds1 : return 3 # "5'UTR"
      elif start > cds2 : return 4 # "3'UTR"
      else : return 5 # "Inside"

if __name__ == '__main__':
  import sys, argparse
  p = argparse.ArgumentParser()
  set_parser(p)
  if len(sys.argv)==1:
    print(p.print_help())
    exit(0)
  run(p.parse_args())


    



from ribotish.zbio import gtf, bam, ribo, stat, exp, tools, orf, fa, interval, io
import math, time, itertools, sys
from os.path import isfile

def help():
  return "Differential TIS calling"
def set_parser(parser):
  #### basic input options ####
  parser.add_argument("-1", type=strlist, dest="tis1paths", required=True, help="Prediction results of group 1 TIS data")
  parser.add_argument("-2", type=strlist, dest="tis2paths", required=True, help="Prediction results of group 2 TIS data")
  parser.add_argument("-a", type=strlist, dest="tis1bampaths", required=True, help="Group 1 TIS enriched riboseq bam files, comma seperated")
  parser.add_argument("-b", type=strlist, dest="tis2bampaths", required=True, help="Group 2 TIS enriched riboseq bam files, comma seperated")
  parser.add_argument("-g", type=str, dest="genepath", required=True, help='Gene annotation file')
  #parser.add_argument("-f", type=str, dest="genomefapath", required=True, help="genome fasta file")
  parser.add_argument("-o", type=str, dest="output", required=True, help="Output result file")
  #### alt input options ####
  parser.add_argument("--tis1para", type=strlist, help="Input offset parameter files for group 1 bam files")
  parser.add_argument("--tis2para", type=strlist, help="Input offset parameter files for group 2 bam files")
  parser.add_argument("--geneformat", type=str, default='auto', help="Gene annotation file format (gtf, bed, gpd, gff, default: auto)")
  parser.add_argument("--maxNH", type=int, default=5, help="Max NH value allowed for bam alignments (default: 5)")
  parser.add_argument("--minMapQ", type=float, default=1, help="Min MapQ value required for bam alignments (default: 1)")
  parser.add_argument("--secondary", action="store_true", help="Use bam secondary alignments")
  parser.add_argument("--paired", action="store_true", help="Reads are paired end")

  parser.add_argument("--l1", type=strlist, dest="tis1labels", default=[], help="Labels for group 1 TIS data")
  parser.add_argument("--l2", type=strlist, dest="tis2labels", default=[], help="Labels for group 2 TIS data")

  parser.add_argument("--nocompatible", action="store_true", help="Do not require reads compatible with transcript splice junctions")
  parser.add_argument("--compatiblemis", type=int, default=2, help="Missed bases allowed in reads compatibility check")
  parser.add_argument("--chrmap", type=str, help="Input chromosome id mapping table file if annotation chr ids are not same as chr ids in bam/fasta files")
  parser.add_argument("--normcomm", action="store_true", help="Use common TISs instead of union TISs for normalization")
  parser.add_argument("--normanno", action="store_true", help="Use only annotated TISs for normalization")

  parser.add_argument("--rnaseq", type=str, help="RNASeq count input")
  parser.add_argument("--scalefactor", type=float, help="Input TIS scale factor of 2/1 (default: auto)")
  parser.add_argument("--rnascale", type=float, help="Input RNASeq scale factor of 2/1 (default: auto)")
  parser.add_argument("--chi2", action="store_true", help="Use chisquare test instead of Fisher's exact test for mRNA referenced test")
  parser.add_argument("--betabinom", action="store_true", help="Use beta-binom test instead of Fisher's exact test for mRNA referenced test")

  parser.add_argument("--export", type=str, help="Export count table for differential analysis with other tools")

  # scatter plot result output
  parser.add_argument("--plotout", type=str, help="Scatter plot output pdf file")
  parser.add_argument("--figsize", type=int2, default=(8,8), help="Scatter plot figure size (default: 8,8)")
  parser.add_argument("--plotma", type=str, help="TIS normalization MA plot output pdf file")

  # other options
  parser.add_argument("--qi", type=int, default=15, help="Index of TIS q value, 0 based (default: 15)")
  parser.add_argument("-f", type=float, dest="foldchange", default=1.5, help="Minimum fold change threshold (default: 1.5)")
  parser.add_argument("--ipth", type=float, default=0.05, help="Input TIS p value threshold (default: 0.05)")
  parser.add_argument("--iqth", type=float, default=0.05, help="Input TIS q value threshold (default: 0.05)")
  parser.add_argument("--opth", type=float, default=0.05, help="Output TIS diff p value threshold (default: 0.05)")
  parser.add_argument("--oqth", type=float, default=0.05, help="Output TIS diff q value threshold (default: 0.05)")
  parser.add_argument("-p", type=int, dest="numProc", default=1, help="Number of processes")
  parser.add_argument("-v", "--verbose", action="count", default=0, help="Increase output verbosity")
  
def strlist(s):
  '''Convert comma seperated file name string to list
  '''
  return s.split(',')

def int2(s):
  '''Convert comma seperated string to tuple
  '''
  lst = eval('['+s+']')
  return tuple(map(int, lst))

try: imap = itertools.imap
except: imap = map

def sig(data):
  '''if data fit thresholds
  '''
  return data[1] < ipth and data[2] < iqth
def get_tis(gp):
  '''get TIS genome position
  '''
  lst = gp.split(':')
  lst2 = lst[1].split('-')
  if lst[2] != '-' : lst[1] = lst2[0]
  else : lst[1] = lst2[-1]
  return ':'.join(lst)
def run(args):
  '''Main function for differential TIS
  '''
  global ipth, iqth, tis1bampaths, tis2bampaths, tis1offdict, tis2offdict, compatible, compatiblemis, paired
  ipth, iqth = args.ipth, args.iqth
  tis1bampaths = args.tis1bampaths
  tis2bampaths = args.tis2bampaths
  ribo.maxNH, ribo.minMapQ, ribo.secondary = args.maxNH, args.minMapQ, args.secondary
  compatible = not args.nocompatible
  compatiblemis = args.compatiblemis
  paired = args.paired
  if len(tis1bampaths) < len(args.tis1paths) or len(tis2bampaths) < len(args.tis2paths) : # == 0 :
    print('Missing bam file input!')
    exit(1)
  if args.chrmap is not None :
    chrmap = {}
    for lst in io.splitIter(args.chrmap, sep=None):
      if len(lst) < 2: continue
      chrmap[lst[0]] = lst[1]
      chrmap[lst[1]] = lst[0]
    bam.chrmap = chrmap
    fa.chrmap = chrmap

  global tis1bampathslist, tis2bampathslist, tis1offdictlist, tis2offdictlist
  tis1bampathslist = [s.split(';') for s in args.tis1bampaths]
  tis2bampathslist = [s.split(';') for s in args.tis2bampaths]
  if args.tis1para is None : tis1paralist = [None] * len(args.tis1paths)
  else : tis1paralist = [s.split(';') for s in args.tis1para]
  if args.tis2para is None : tis2paralist = [None] * len(args.tis2paths)
  else : tis2paralist = [s.split(';') for s in args.tis2para]

  tis1offdictlist = [find_offset(bampaths, para) for bampaths, para in zip(tis1bampathslist, tis1paralist)]
  tis2offdictlist = [find_offset(bampaths, para) for bampaths, para in zip(tis2bampathslist, tis2paralist)]

  if len(args.tis1labels) < len(args.tis1paths) : 
    for i in range(len(args.tis1labels), len(args.tis1paths)):
      args.tis1labels.append(args.tis1paths[i])
  if len(args.tis2labels) < len(args.tis2paths) :
    for i in range(len(args.tis2labels), len(args.tis2paths)):
      args.tis2labels.append(args.tis2paths[i])
  title = args.tis1labels + args.tis2labels
  tis_title = ['TIS_'+lab for lab in title]
  rna_title = ['RNA_'+lab for lab in title]
  l = len(title)

  if args.rnaseq is not None :
    if args.verbose : print("Loading RNASeq data...")
    rna_profile = exp.Profile()
    for lst in io.splitIter(args.rnaseq):
      try : values = list(map(int, lst[1:]))
      except : 
        try : 
          values = list(map(float, lst[1:]))
          print('Error: RNASeq data should be integers {}.'.format(lst))
          sys.exit(1)
        except ValueError: pass
        continue
      m = min(values)
      if m < 0 : 
        print('Error: RNASeq data should be non-negative integers {}.'.format(lst))
        sys.exit(1)
      e = exp.Exp(lst[0], values)
      rna_profile.add_exp(e)
    if args.verbose : print("{} genes.".format(len(rna_profile)))

  if args.verbose : print("Loading {} TIS data...".format(l))
  gname, gpos, gsig = {}, {}, {}
  tall = [] # {}, {}
  gid = {}
  anno = {} # annotated TIS
  for i, fname in enumerate(args.tis1paths + args.tis2paths) :
    n = 0
    tdata = {}
    for lst in io.splitIter(fname) :
      try : tis = (lst[1], int(lst[6]))
      except : continue
      gid[lst[1]] = lst[0]
      cnt, pval, qval =int(lst[10]), float(lst[11]), float(lst[args.qi])
      tdata[tis] = cnt, pval, qval
      lst[4] = get_tis(lst[4])
      if lst[8] == 'Annotated' : anno[lst[4]] = 1 # genome position
      if sig(tdata[tis]) : 
        n += 1
        #lst[4] = get_tis(lst[4])
        gname[tis] = '\t'.join(lst[:9]) # information for the TIS
        gpos[tis] = lst[4]
        if tis not in gsig : gsig[tis] = [0] * len(tis_title)
        gsig[tis][i] = 1
    if args.verbose : print("{} TISs in {}.".format(n, fname))
    tall.append(tdata)

  profile = exp.Profile()
  profile2 = exp.Profile() # uniq TISs for TMM
  trans_for_bam = {} # TIS genes need to be analyzed
  es = {}
  uniq_gpos = {}
  for tis in gname : # t1 :
    values = []
    for i, tdata in enumerate(tall) : 
      if tis not in tdata : 
        if tis[0] not in trans_for_bam : trans_for_bam[tis[0]] = [{} for j in range(l)]
        trans_for_bam[tis[0]][i][tis[1]] = None 
        values.append(0) #(None)
      else : values.append(tdata[tis][0])
    if args.rnaseq is not None : 
      if tis[0] in rna_profile.exps : values += rna_profile.exps[tis[0]].data # trans level
      elif gid[tis[0]] in rna_profile.exps : values += rna_profile.exps[gid[tis[0]]].data # gene level
      else :
        print('Warning: transcript {} {} is not found in RNA file!'.format(gid[tis[0]], tis[0]))
        values += [0] * l # len(title)

    e = exp.Exp(gname[tis], values)
    e.tis = tis
    es[tis] = e
    profile.add_exp(e)
    if gpos[tis] not in uniq_gpos :
      if args.normanno and gpos[tis] not in anno : continue
      if args.normcomm : 
        for tdata in tall :
          if tis not in tdata or not sig(tdata[tis]) : break
        else : profile2.add_exp(e)
      #elif args.normanno : 
        #if gpos[tis] in anno : profile2.add_exp(e)
      else : 
        profile2.add_exp(e)
      uniq_gpos[gpos[tis]] = 1

  if args.verbose : print("{} TISs in total.".format(len(profile)))
  if not args.normcomm :
    uns = [0] * len(tis_title)
    for i in range(len(tis_title)) :
      uns[i] = len([i for e in profile2 if gsig[e.tis][i] == 1])
    m = min(uns)
    elst = list(profile2.exps.values())
    profile3 = exp.Profile()
    for i in range(len(tis_title)) :
      for e in elst : e.value[0:2] = [gsig[e.tis][i], e.data[i]]
      elst.sort(reverse = True)
      for j in range(m) :
        if elst[j].id not in profile3.exps : profile3.add_exp(elst[j])
    profile2 = profile3

  if args.verbose : print ("Reading bams...")
  trans_iter = io.transIter(args.genepath, fileType = args.geneformat, verbose = args.verbose, filt = trans_for_bam)
  para_iter = transPara(trans_iter, trans_for_bam)
  if args.numProc <= 1 : pred_iter = imap(_get_tis, para_iter)
  else : 
    from multiprocessing import Pool
    pool = Pool(processes = args.numProc - 1)
    pred_iter = pool.imap_unordered(_get_tis, para_iter, chunksize = 5)
  for result in pred_iter: 
    tid, pos_cnt = result # r1, r2 = result
    for i, pc in enumerate(pos_cnt):
      for pos in pc :
        tis = tid, pos
        es[tis].data[i] = pc[pos]

  if len(args.tis1paths) > 1 or len(args.tis2paths) > 1 or args.export is not None : 
    if args.export is None : args.export = 'tisdiff_export.txt'
    if args.verbose : print('Export TIS counts table to {}.'.format(args.export))
    exfile = open(args.export, 'w')
    if args.rnaseq is None : exfile.write(io.tabjoin('TIS', tis_title)+'\n') # args.tis1labels, args.tis2labels)+'\n')
    else :  exfile.write(io.tabjoin('TIS', tis_title, rna_title)+'\n')
    for tis in gname:
      s = '{}_{}_{}\t'.format(tis[0], tis[1], gpos[tis])
      s += io.tabjoin(es[tis].data)
      exfile.write(s+'\n')
    return
  if args.scalefactor is not None : scale = args.scalefactor
  else :
    if args.verbose : print ('Estimate scale factor...')
    f = profile2.TMM(i1 = 0, i2 = 1)
    if args.verbose : print ('TIS TMM log2 f = {}'.format(f))
    scale = 2 ** (-f)
  if args.rnaseq is not None : 
    if args.rnascale is not None : scale_r = args.rnascale
    else :
      fr = rna_profile.TMM(i1 = 0, i2 = 1) # for only one replicate
      if args.verbose : print ('RNASeq TMM log2 f = {}'.format(fr))
      scale_r = 2 ** (-fr)

  if args.verbose : print ('Diff test...')

  exps = profile.exps.values()
  for e in exps:
    if args.rnaseq is None :
      p = 1 / (scale + 1)
      x, y = e.data[0], e.data[1] # [2]
      n = x + y
      if x == 0 : fc, alt = 'INF', 'less'
      elif y == 0 : fc, alt = 0, 'greater'
      else : 
        fc = y / (1.0 * x * scale) # / y
        if scale * x <= y : alt = 'less'
        else : alt = 'greater'
      pv = stat.binom_test(n, x, p = p, alt = alt)
    else : 
      x, y, r1, r2 = e.data[0:4]
      if x == 0 : fc, alt = 'INF', 'less'
      elif y == 0 or r1 == 0: fc, alt = 0, 'greater'
      elif r2 == 0 : fc, alt = 'INF', 'less'
      else : 
        fc = y / (1.0 * x * scale) / (r2 / (1.0 * r1 * scale_r))
        if fc >= 1 : alt = 'less' # test x
        else : alt = 'greater'
      if args.chi2 : pv = ribo.TIStest_chi2(x, y, r1, r2, scale, scale_r, alt = alt)
      elif args.betabinom : pv = ribo.TIStest_betaBinom(x, y, r1, r2, scale, scale_r, alt = alt)
      else : pv = ribo.TIStest_FisherExact(x, y, r1, r2, scale, scale_r, alt = alt)
    pv *= 2 # two tailed
    if pv > 1 : pv = 1
    e.data.append(fc)
    e.data.append(pv) 

  result = profile.BHcorrection(-1, append = True) # (5) 

  if args.verbose : print ('Output...')
  outfile = open(args.output, 'w')
  s = "Gid\tTid\tSymbol\tGeneType\tGenomePos\tStartCodon\tStart\tStop\tTisType\t"
  s += '\t'.join(tis_title)
  if args.rnaseq is not None : s += '\t' + '\t'.join(rna_title)
  s += '\tFoldChange\tDiffPvalue\tDiffQvalue\n'
  outfile.write(s)
  for e in profile :
    fc = e.data[-3]
    e.is_q = e.is_fc = True
    if fc != 'INF' and fc != 0 and max(fc, 1/fc) < args.foldchange : 
      e.is_fc = False
    if e.data[-2] > args.opth or e.data[-1] > args.oqth : 
      e.is_q = False
    if e.is_q and e.is_fc : 
      outfile.write(str(e)+'\n')

  # Plot
  if args.plotout is not None :
    if args.verbose : print ("Ploting...")
    from zbio import plot
    plot.figure(figsize = args.figsize)
    if args.rnaseq is not None : 
      qd1 = [math.log(e.data[0]+1,2) - math.log(e.data[2]+1,2) for e in exps if e.is_q and e.is_fc]
      qd2 = [math.log(e.data[1]+1,2) - math.log(e.data[3]+1,2) for e in exps if e.is_q and e.is_fc]
      pd1 = [math.log(e.data[0]+1,2) - math.log(e.data[2]+1,2) for e in exps if e.is_q and not e.is_fc]
      pd2 = [math.log(e.data[1]+1,2) - math.log(e.data[3]+1,2) for e in exps if e.is_q and not e.is_fc]
      nd1 = [math.log(e.data[0]+1,2) - math.log(e.data[2]+1,2) for e in exps if not e.is_q]
      nd2 = [math.log(e.data[1]+1,2) - math.log(e.data[3]+1,2) for e in exps if not e.is_q]
    else :
      qd1 = [math.log(e.data[0]+1,2) for e in exps if e.is_q and e.is_fc]
      qd2 = [math.log(e.data[1]+1,2) for e in exps if e.is_q and e.is_fc]
      pd1 = [math.log(e.data[0]+1,2) for e in exps if e.is_q and not e.is_fc]
      pd2 = [math.log(e.data[1]+1,2) for e in exps if e.is_q and not e.is_fc]
      nd1 = [math.log(e.data[0]+1,2) for e in exps if not e.is_q]
      nd2 = [math.log(e.data[1]+1,2) for e in exps if not e.is_q]
    plot.scatter(qd1, qd2, alpha=0.1, edgecolors='none', color='r', label='q < {} & FC > {}'.format(args.oqth, args.foldchange))
    plot.scatter(pd1, pd2, alpha=0.1, edgecolors='none', color='y', label='q < {} & FC <= {}'.format(args.oqth, args.foldchange))
    plot.scatter(nd1, nd2, alpha=0.1, edgecolors='none', color='g', label='q >= {}'.format(args.oqth))
    plot.legend(loc='upper left', frameon=False)
    plot.xlabel(title[0])
    plot.ylabel(title[1])
    if args.rnaseq is not None : d = (fr - f) / 2 
    else : d = - f / 2
    m1 = max(min(qd1+pd1+nd1), min(qd2+pd2+nd2))
    m2 = min(max(qd1+pd1+nd1), max(qd2+pd2+nd2))
    plot.plot([m1-d,m2-d],[m1+d,m2+d], color='k', linestyle = ':')
    d2 = d - math.log(args.foldchange, 2) / 2
    plot.plot([m1-d2,m2-d2],[m1+d2,m2+d2], color='r', linestyle = ':')
    d2 = d + math.log(args.foldchange, 2) / 2
    plot.plot([m1-d2,m2-d2],[m1+d2,m2+d2], color='r', linestyle = ':')
    plot.save(args.plotout)

    if args.plotma is not None :
      exps = profile2.exps.values()
      plot.figure(figsize = args.figsize)
      plot.axhline(f)
      ms = [e.M for e in exps if hasattr(e, 'select') and e.select]
      aa = [e.A for e in exps if hasattr(e, 'select') and e.select]
      plot.scatter(aa, ms, alpha=0.1, edgecolors='none',color='r')
      ms = [e.M for e in exps if hasattr(e, 'select') and not e.select]
      aa = [e.A for e in exps if hasattr(e, 'select') and not e.select]
      plot.scatter(aa, ms, alpha=0.1, edgecolors='none',color='b')
      plot.save(args.plotma)


    
def transPara(trans_iter, trans_for_bam) : #g1, g2):
  '''Generate parameters (trans, g1t, g2t) for function _get_tis()
  '''
  for t in trans_iter :
    if t.id in trans_for_bam : yield t, trans_for_bam[t.id]

def _get_tis(ps) : 
  '''get non-significant TIS counts
  '''
  t, pos_cnt = ps # g1t, g2t = ps
  bamlist = tis1bampathslist + tis2bampathslist
  offlist = tis1offdictlist + tis2offdictlist
  for i, pc in enumerate(pos_cnt) :
    if len(pc) == 0 : continue
    ttis = ribo.multiRibo(t, bamlist[i], offdict=offlist[i], compatible=compatible, mis=compatiblemis, paired=paired)
    for pos in pc : pc[pos] = ttis.cnts[pos]
  return t.id, pos_cnt

def find_offset(bampaths, para):
  '''Get offset data for given bam data
  '''
  offlist = [None] * len(bampaths)
  for i, path in enumerate(bampaths):
    if para is None or i > len(para)-1 or para[i] == '' : path = path + '.para.py'
    else : path = para[i]
    if isfile(path) : 
      vessel = {}#'offdict': None}
      exec(open(path).read(), vessel) # execfile(path, vessel)
      offlist[i] = vessel['offdict']
  for i, od in enumerate(offlist):
    if od is None :
      print('No offset parameter file found for %s. Using default offset (12). ' %  bampaths[i])
  return offlist
    



'''
Expression profile processing
Copyright (c) 2016 Peng Zhang <zhpn1024@163.com>
'''
def cmp3(a, b):
  ''' cmp for python3'''
  if a is None:
    if b is None: return 0
    else: return -1
  return bool(a > b) - bool(a < b)

import math
class Exp(): #values for one gene/trans/probe
  '''expression values for one gene/transcript
  '''
  def __init__(self, id, data, anno = ''): #sample and expression list
    self.id = id
    #self.sample = sample
    self.data = data
    self.anno = anno
    self.value = [] #sort index
  def __str__(self, showanno = False, sep = '\t'):
    s = self.id + sep
    if showanno : s += self.anno + sep
    return s + sep.join(map(str, self.data))
  def string(self, showanno = False, sep = '\t'):
    return self.__str__(showanno, sep)
  def __repr__(self):
    return self.headerline() + "\n" + str(self)
  def __len__(self):
    return len(self.data)
  def __cmp__(self, other):
    c = 0
    for i in range(len(self.value)):
      c = c or cmp(self.value[i], other.value[i])
      if c != 0: break
    return c
  def cmp(self, other):
    c = 0
    for i in range(len(self.value)):
      c = c or cmp3(self.value[i], other.value[i])
      if c != 0: break
    return c
  def __eq__(self, other):
    return self.cmp(other) == 0
  def __lt__(self, other):
    return self.cmp(other) < 0
  def __gt__(self, other):
    return self.cmp(other) > 0

  def headerline(self, showanno = False, sep='\t'):# Header string
    sep=str(sep)
    s = 'id' + sep
    if showanno : s += 'anno' # + sep
    #s += sep.join(map(str, self.sample))
    return s
  
class Gene:
  def __init__(self, gid):
    self.id = gid
    #exp.__init__(self, gid, sample, data)
    self.trans = []
  @property
  def gid(self):
    return self.id
  def add_trans(self, t):
    self.trans.append(t)
  def headerline(self,sep='\t'):# not finished!
    sep=str(sep)
    if len(self.trans) == 0:
      return 'gid' + sep + sep.join(map(str, self.sample))
    else:
      return 'gid' + sep + 'tid' + sep + sep.join(map(str, self.sample))
  
def subarr(lst, ids = [], head = 0): # selected items in the list
  if len(ids) == 0 : return lst[head:]
  return [lst[i] for i in ids]
  #a = []
  #for i in ids:
        #a.append(lst[i])
  #return a
def gtexp_iter(expfile, gi = 0, ti = 1, sep = '\t', ei = [], skip = 0):
  for i in range(skip) : l = next(expfile)
  #if len(sample) == 0 :
    #l = next(expfile) #.next()
    #lst = l.strip('\n').split(sep)
    #sample = subarr(lst, ei, ti+1)
  for l in expfile:
    lst = l.strip('\n').split(sep)
    n = len(lst)
    gid = lst[gi]
    tid = lst[ti]
    data = list(map(float, subarr(lst, ei, ti+1)))
    t = Exp(tid, data, anno = gid)
    t.gid = gid
    yield t
def gtexp_load(expfile, gi = 0, ti = 1, sep = '\t', ei = [], skip = 0):
  gs = {}
  for t in gtexp_iter(expfile, gi, ti, sep, ei, skip):
    if t.gid not in gs:
      gs[t.gid] = gene(t.gid)
    gs[t.gid].add_trans(t)
  return gs

def exp_iter(expfile, ii = 0, dsi = -1, annoi = -1, sep = '\t', ei = [], skip = 0, innerskip = []):
  if dsi < 0 : dsi = max(ii, annoi) + 1 # supposed data start id
  for i in range(skip) : l = next(expfile) #.expfile.next()
  if False and header :
    l = next(expfile) #.expfile.next()
    i += 1
    lst = l.strip('\n').split(sep)
    sample = subarr(lst, ei, dsi)
  for l in expfile:
    i += 1
    if i in innerskip : continue
    lst = l.strip('\n').split(sep)
    n = len(lst)
    id = lst[ii]
    anno = ''
    if annoi >= 0 : anno = lst[annoi]
    data = list(map(float, subarr(lst, ei, dsi)))
    e = exp(id, data, anno)
    yield e

class Profile():
  '''expression profile of a list of genes
  '''
  def __init__(self):
    self.exps = {}
    self.ids = []
  def add_exp(self, e):
    self.exps[e.id] = e
    self.ids.append(e.id)
  def __len__(self):
    return len(self.exps)
  def __iter__(self):
    for eid in self.ids:
      yield self.exps[eid]
  def __getitem__(self, i):
    return self.exps[self.ids[i]]
  def BHcorrection(self, pid = -1, total = -1, append = False):
    lst = list(self.exps.values())
    n = len(lst)
    if total < 0: total = n
    for e in lst:
      #if len(e.value) < 1: e.value.append(1)
      e.value = [e.data[pid]]
    lst.sort()
    qc = 1
    for i in range(n-1, -1, -1):
      if lst[i].value[0] is None : q = None
      else : 
        q = float(lst[i].value[0]) * total / (i+1)
        if q > qc : q = qc
      lst[i].q = q
      if append : lst[i].data.append(q)
      qc = q
    return lst
  def write(self, outfile, showanno = False, sep = '\t'):
    for eid in self.ids:
      outfile.write(self.exps[eid].string(showanno, sep) + '\n')
      #showanno = False
  def TMM(self, i1 = 0, i2 = 1, mtrim = 0.3, atrim = 0.1): # The Trimmed Mean of M-values by edgeR, return log2 scale factor
    exps = [] # list(self.exps.values())
    #n = len(exps)
    #nmt = int(mtrim * n) + 1 # m trim 0.3
    #nat = int(atrim * n) + 1 # a trim 0.1
    N1, N2 = 0, 0
    #s = 0
    for eid in self.exps:
      e = self.exps[eid]
      N1 += e.data[i1]
      N2 += e.data[i2]
      if e.data[i1] <= 0 or e.data[i2] <= 0 : continue
      e.M = math.log(1.0 * e.data[i1] / e.data[i2], 2)
      e.A = 0.5 * math.log(e.data[i1] * e.data[i2], 2)
      #e.V = 1.0 / e.data[i1] + 1.0 / e.data[i2]
      #s += e.M
      #N1 += e.data[i1]
      #N2 += e.data[i2]
      #e.data += [m, a, v]
      exps.append(e)
      e.select = True
      e.value[0:1] = [e.M] ### sort1 = m
    #s /= len(exps)
    #print('mean of M: {}'.format(s))
    exps.sort()
    n = len(exps)
    nmt = int(mtrim * n) + 1 # m trim 0.3
    nat = int(atrim * n) + 1 # a trim 0.05

    #print('median of M: {}'.format(exps[int(len(exps))/2].M))
    for i in range(nmt): exps[i].select = False
    for i in range(n-nmt, n) : exps[i].select = False
  
    for e in exps: e.value[0:1] = [e.A] # sort2 = a
    exps.sort()
    for i in range(nat): exps[i].select = False
    for i in range(n-nat, n) : exps[i].select = False

    s = w = 0
    for e in exps:
      if not e.select : continue
      e.V = 1.0 / e.data[i1] + 1.0 / e.data[i2] - 1.0 / N1 - 1.0 / N2 # weight
      s += e.M / e.V
      w += 1 / e.V
    f = s / w
    #print f, s, w, n
    return f

class ReadDict(dict):
  '''dict for read counts
  read count -> times of the count appear
  '''
  def __init__(self, d = {}):
    dict.__init__(self)
    for i in d: self[i] = d[i]
  def sum(self):
    s = 0
    for i in self: s += i * self[i]
    return s
  def size(self):
    #s = 0 
    #for i in self: s += self[i]
    return sum(self.values())
  def value(self, n, default = 0):
    '''allow non-exist keys
    '''
    if n in self : return self[n]
    else : return default
  def mean(self):
    return 1.0 * self.sum() / self.size()
  def quantile(self, r = 0.5):
    size = self.size()
    size *= r
    #ks = list(self.keys())
    #ks.sort()
    ks = sorted(self)
    maxi = len(ks) - 1
    for i, k in enumerate(ks):
      if i == maxi : return k
      size -= self[k]
      if size < 0 : return k
      elif size == 0 : return (k + ks[i+1])/2.0
    return ks[-1]
  def geomean(self, add = 1):
    s = 0
    for i in self:
      s += math.log(i+add, 2) * self[i]
    s /= self.size()
    return 2 ** s - add
  def median(self):
    return self.quantile(0.5)
  def record(self, read, n = 1):
    if read not in self: self[read] = 0
    self[read] += n
  def merge(self, other):
    for read in other : self.record(read, other[read])
  def string(self):
    ks = sorted(self)
    s = '{'
    for k in ks:
      s += "%d:%d, " % (k, self[k])
    s = s.rstrip(', ') + '}'
    return s

        



'''
Some plot functions
Copyright (c) 2016 Peng Zhang <zhpn1024@163.com>
'''

import matplotlib
matplotlib.use('pdf')
matplotlib.rcParams['pdf.fonttype'] = 42
matplotlib.rcParams['ps.fonttype'] = 42
matplotlib.rcParams['svg.fonttype'] = 'none'
from matplotlib.pylab import *

def plotTrans(t, ypos = 0, intv = None, r = [0.1, 0.3], color = 'blue',rid = -0.5):
  '''plot transcript
  '''
  plot([t.start,t.stop],[ypos,ypos],color=color)
  # arrows
  x, y = [], []
  if intv is None : intv = len(t) / 20
  for i in range(t.start+intv//2, t.stop-intv//3, intv):
    x.append(i)
    y.append(ypos)
  arr = '>'
  if t.is_reverse() : arr = "<"
  plot(x,y,'w'+arr)
  # blocks
  x = [[],[]]
  y = [[],[]]
  orf = t(start = t.thick_start, stop = t.thick_stop)
  for e in t.exons:
    for es in e - orf : # UTR 
      x[0].append(es.start)
      y[0].append(len(es))
    for ei in e.intersect(orf): # CDS
      x[1].append(ei.start)
      y[1].append(len(ei))
  bar(x[0],[r[0]*2]*len(x[0]),width=y[0],bottom=ypos-r[0],edgecolor=color,color=color, align='edge')
  bar(x[1],[r[1]*2]*len(x[1]),width=y[1],bottom=ypos-r[1],edgecolor=color,color=color, align='edge')
  text((t.start+t.stop)//2, ypos+rid, t.id)

def save(filename):
  savefig(filename, transparent=True)

def riboShow(ax, trans, cnts, start = 0, stop = -1, ymax = None, scale = 1, col = ['r','g','b'], title = '', showlegend = False, showframe = True, bottom = 0.8, height = 0.1):
  '''plot riboseq profile
  '''
  if stop < start : stop = trans.cdna_length()
  rlen = stop - start
  lx = [[], [], []]
  ly = [[], [], []]
  m = 1
  for j in range(rlen):
    p = j + start
    if cnts[j + start] > 0 : 
      i = p % 3
      lx[i].append(j)
      y = cnts[p] * scale
      if ymax is not None and ymax > 0 and y > ymax: y = ymax
      ly[i].append(y)
      if m < y : m = y
  ylab = 'Count'
  if scale != 1 : ylab = 'Scaled count'
  if ymax is None or ymax < 0 : ymax = m
  for i in range(3):
    ax.bar(lx[i], ly[i], color=col[i], width=1, edgecolor=col[i], log=False, alpha=0.4, label='Frame '+str(i+1), align='edge')
  
  [ax.spines[side].set_visible(False) for side in ('right','top','bottom')]
  ax.yaxis.set_ticks_position('left')
  ax.xaxis.set_ticks_position('bottom')
  ax.set_xlim((0, rlen))
  ax.set_ylim((0, ymax))
  ax.label_outer() # setp(ax.get_xticklabels(), visible=False)
  ax.set_ylabel(ylab)
  ax.set_title(title)
  if showlegend :
    try : ax.legend(loc='best', frameon=False)
    except : pass
  if not showframe : return
  fx = [[],[],[]]
  fy = [[],[],[]]
  fw = [[],[],[]]
  for i in range(3):
    last = False
    for j in range(i, stop, 3) :
      if j < start : continue
      top = True
      if cnts[j] <= 0 : top = False
      if j-1 >= 0 and cnts[j] <= cnts[j-1] : top = False
      if j+1 < stop and cnts[j] <= cnts[j+1] : top = False
      if top : ##
        if last : tw += 3
        else : tx, tw = j-start, 1
        last = True
      else : 
        if last and tw > 3 :
          fx[i].append(tx)
          fy[i].append(ymax * height)
          fw[i].append(tw)
        last = False
    if last and tw > 3 :
      fx[i].append(tx)
      fy[i].append(ymax * 0.1)
      fw[i].append(tw)
  for i in range(3):
    ax.bar(fx[i], fy[i], color=col[i], bottom = ymax * bottom,width=fw[i], alpha=0.2, linewidth = 0, align='edge')
  
def orfShow(ax, orfs, start = 0, stop = -1, col = ['r','g','b'], cds = [None, None], title = 'Potential ORFs in 3 reading frames', alt = True, morecds = None, morecdsbox = False, morecdslabel = None, markpept = None):
  '''plot possible ORFs
  '''
  if stop < start : stop = trans.cdna_length()
  rlen = stop - start
  # ORF in 3 frames
  lx = [[],[],[]]
  ly = [[],[],[]]
  orf_s = []
  for o in orfs:
    if not alt and len(o.starts) == 0 : continue
    if 0 <= o.stop <= start or o.start(alt=alt) >= stop : continue
    orf_s.append(o)
    o1 = o.start(alt=alt) - start
    if o1 < 0: o1 = 0
    lx[o.frame-1].append(o1) # (o.start(alt=alt) - start)
    if o.has_stop():
      o2 = min(stop, o.stop) - max(o.start(alt=alt), start)
    else:
      o2 = stop - max(o.start(), start)
    ly[o.frame-1].append(o2)
    #if o.has_stop(): ly[o.frame-1].append(o.stop-o.start(alt=alt))
    #else : ly[o.frame-1].append(rlen)
  for i in range(3):
    ax.bar(lx[i], [0.2]*len(lx[i]), color=col[i], bottom=2-i+0.4, width=ly[i], alpha=0.4, linewidth=0, align='edge')
  # annotated ORF
  if cds[0] is not None and not (cds[0]>stop or cds[1]< start): 
    i = cds[0] % 3
    newcds = [c - start for c in cds]
    if newcds[0] < 0: newcds[0] = 0
    if cds[1] > stop: newcds[1] = rlen
    ax.text(max(newcds[0],0), 2-i+0.8, 'Annotated ORF', color=col[i])
    ax.bar(newcds[0], [0.4] ,color=col[i], bottom=2-i+0.3, width=newcds[1]-newcds[0], alpha=0.3, edgecolor=col[i], linewidth=2, align='edge')
  if morecds is not None:
    for j, mc in enumerate(morecds):
      if mc[0]>stop or mc[1]< start: continue
      i = mc[0] % 3
      newcds = [c - start for c in mc]
      if newcds[0] < 0: newcds[0] = 0
      if mc[1] > stop: newcds[1] = rlen
      #ax.text(newcds[0], 2-i+0.8, 'Annotated ORF', color=col[i])
      ax.bar(newcds[0], [0.4] ,color=col[i], bottom=2-i+0.3, width=newcds[1]-newcds[0], alpha=0.3, edgecolor=col[i], linewidth=2, align='edge')
      if morecdsbox:
        ax.bar(newcds[0], [0.4] ,color='None', bottom=2-i+0.3, width=newcds[1]-newcds[0], alpha=0.8, fill=False, edgecolor=col[i], linewidth=2, align='edge')
      if morecdslabel is not None:
        ax.text(max(newcds[0],0), 2-i+0.8, morecdslabel[j], color=col[i])
  if markpept is not None:
    for j, mc in enumerate(markpept):
      if mc[0]>stop or mc[1]< start: continue
      i = mc[0] % 3
      newcds = [c - start for c in mc]
      if newcds[0] < 0: newcds[0] = 0
      if mc[1] > stop: newcds[1] = rlen
      ax.bar(newcds[0], [0.2], color='k', bottom=2-i+0.4, width=newcds[1]-newcds[0], alpha=0.8, linewidth=0, align='edge')
  # start & stop codons
  lx = [[],[],[]]
  ly = [[],[],[]]
  lz = [[],[],[]]
  for o in orf_s:
    lx[o.frame-1] += [s - start for s in o.starts if start<=s<stop]
    if alt : ly[o.frame-1] += [s - start for s in o.altstarts if start<=s<stop] # orf.altstarts
    if o.has_stop() and start<=o.stop-3<stop: lz[o.frame-1] += [o.stop - 3 - start]
  for i in range(3):
    ax.bar(ly[i], [0.4]*len(ly[i]), color='yellow', bottom=2-i+0.3, width=3, alpha=0.6, edgecolor='yellow', align='edge')
    ax.bar(lx[i], [0.4]*len(lx[i]), color='lime', bottom=2-i+0.3, width=3, alpha=1, edgecolor='lime', align='edge')
    ax.bar(lx[i], [0.04]*len(lx[i]), color='w', bottom=2-i+0.48, width=3, edgecolor='w', align='edge')
    ax.bar(lz[i], [0.4]*len(lz[i]), color='red', bottom=2-i+0.3, width=3, alpha=1, edgecolor='red', align='edge')
    ax.bar(lz[i], [0.04]*len(lz[i]), color='k', bottom=2-i+0.48, width=3, edgecolor='k', align='edge')
  ax.set_title(title)
  
